# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.database import database_service

from ecl import exceptions
from ecl import resource2
from ecl import utils


class Instance(resource2.Resource):
    resource_key = 'instance'
    resources_key = 'instances'
    base_path = '/instances'
    service = database_service.DatabaseService()

    # capabilities
    allow_create = True
    allow_get = True
    allow_delete = True
    allow_list = True

    allow_update = False

    _query_mapping = resource2.QueryParameters()

    #: ID of the server
    id = resource2.Body('id')

    #: Timestamp of when the instance was created.
    created_at = resource2.Body('created')

    #: The datastore property returned as instence property.
    datastore = resource2.Body('datastore', type=dict)

    #: The flavor property returned from instance.
    flavor = resource2.Body('flavor', type=dict)

    #: The flavor reference, as a ID or full URL,
    #: in case create instane.
    flavor_id = resource2.Body('flavorRef')

    #: An ID representing the host of this instance.
    hostname = resource2.Body('hostname')

    #: A list of dictionaries holding links relevant to this instance.
    links = resource2.Body('links', type=list)

    #: Name of the instance
    name = resource2.Body('name')

    #: Region of the instance
    region = resource2.Body('region')

    #: The state this instance is in.
    status = resource2.Body('status')

    #: Timestamp of when this instance was last updated.
    updated_at = resource2.Body('updated')

    #: The volume property returned from instance.
    volume = resource2.Body('volume', type=dict)

    #: A nic definition object. Required parameter when there are multiple
    #: networks defined for the tenant.
    nics = resource2.Body('nics', type=dict)

    #: Databases list of instance.
    databases = resource2.Body('databases', type=list)

    #: Users list of instance.
    users = resource2.Body('users', type=list)

    #: Backup window of instance.
    backup_window = resource2.Body('backup_window')

    #: Maintenance window of instance.
    maintenance_window = resource2.Body('maintenance_window')

    #: Backup retension period of this instance.
    backup_retention_period = resource2.Body('backup_retention_period',
                                             type=int)

    #: Restore Point
    restore_point = resource2.Body('restorePoint', type=dict)

    #: Restoreable Time
    restorable_time = resource2.Body('restorable_time')

    #: Endpoints
    endpoints = resource2.Body('endpoints', type=list)

    #: Availability Zone
    availability_zone = resource2.Body('availability_zone')

    @classmethod
    def find(cls, session, name_or_id, ignore_missing=False, **params):
        """Find a resource by its name or id.

        :param session: The session to use for making this request.
        :type session: :class:`~ecl.session.Session`
        :param name_or_id: This resource's identifier, if needed by
                           the request. The default is ``None``.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :param dict params: Any additional parameters to be passed into
                            underlying methods, such as to
                            :meth:`~ecl.resource2.Resource.existing`
                            in order to pass on URI parameters.

        :return: The :class:`Resource` object matching the given name or id
                 or None if nothing matches.
        :raises: :class:`ecl.exceptions.DuplicateResource` if more
                 than one resource is found for this request.
        :raises: :class:`ecl.exceptions.ResourceNotFound` if nothing
                 is found and ignore_missing is ``False``.
        """
        # Try to short-circuit by looking directly for a matching ID.

        data = cls.list(session, **params)

        result = cls._get_one_match(name_or_id, data)
        if result is not None:
            return result

        if ignore_missing:
            return None
        raise exceptions.ResourceNotFound(
            "No %s found for %s" % (cls.__name__, name_or_id))



# -*- coding: utf-8 -*-

from ecl.network import network_service
from ecl import resource2


class PhysicalPort(resource2.Resource):
    resource_key = 'physical_port'
    resources_key = 'physical_ports'
    service = network_service.NetworkService("v2.0")
    base_path = '/' + service.version + '/physical_ports'

    # capabilities
    allow_get = True
    allow_list = True

    # properties
    #: Physical port description.
    description = resource2.Body('description')
    #: Physical port unique id.
    id = resource2.Body('id')
    #: Physical port name.
    name = resource2.Body('name')
    #: The type of traffic that port will be used.
    plane = resource2.Body('plane')
    #: Ranges of allowed VLAN tags.
    segmentation_ranges = resource2.Body('segmentation_ranges')
    #: The ID of service that owns the physical port.
    service_id = resource2.Body('service_id')
    #: The type of physical port service owner.
    service_owner = resource2.Body('service_owner')
    #: The Physical Port status.
    status = resource2.Body('status')
    #: Port tags.
    tags = resource2.Body('tags')
    #: The owner name of physical port.
    tenant_id = resource2.Body('tenant_id')



# -*- coding: utf-8 -*-


import six

from ecl import exceptions
from ecl.tests.functional import base


class TestEIC(base.BaseFunctionalTest):

    @classmethod
    def setUpClass(cls):
        super(TestEIC, cls).setUpClass()
        meics = list(cls.conn.connectivity.meics())
        cls.one_meic = None
        cls.one_eic = None
        if meics and len(meics) > 0:
            cls.one_meic = meics[0]
        if cls.one_meic is None:
            return

        eics = list(cls.conn.connectivity.eics(cls.one_meic.mcic_id))
        if eics and len(eics) > 0:
            cls.one_eic = eics[0]

    def test_eics(self):
        eics = list(self.conn.connectivity.eics(self.one_meic.mcic_id))
        self.assertGreaterEqual(len(eics), 0)
        for eic in eics:
            # print eic.to_dict()
            self.assertIsInstance(eic.cic_id, six.string_types)
            if eic.cic_name is not None:
                self.assertIsInstance(eic.cic_name, six.string_types)
            if eic.cic_status is not None:
                self.assertIsInstance(eic.cic_status, six.string_types)
            if eic.network_id is not None:
                self.assertIsInstance(eic.network_id, six.string_types)
            if eic.network_name is not None:
                self.assertIsInstance(eic.network_name, six.string_types)
            if eic.server_segment_name is not None:
                self.assertIsInstance(eic.server_segment_name, six.string_types)
            if eic.server_segment_nbr is not None:
                self.assertIsInstance(eic.server_segment_nbr, int)
            if eic.bandwidth is not None:
                self.assertIsInstance(eic.bandwidth, int)

    def test_get_eic_by_id(self):
        eic = self.conn.connectivity.get_eic(self.one_meic.mcic_id,
                                             self.one_eic.cic_id)
        self.assertEqual(eic.cic_id, self.one_eic.cic_id)
        self.assertEqual(eic.cic_name, self.one_eic.cic_name)
        self.assertEqual(eic.cic_status, self.one_eic.cic_status)
        self.assertEqual(eic.network_id, self.one_eic.network_id)
        self.assertEqual(eic.network_name, self.one_eic.network_name)
        self.assertEqual(eic.server_segment_name, self.one_eic.server_segment_name)
        self.assertEqual(eic.server_segment_nbr, self.one_eic.server_segment_nbr)
        self.assertEqual(eic.bandwidth, self.one_eic.bandwidth)

    def test_create_eic(self):
        eic_name = self.one_meic.mcic_name + "-xxx"
        network_id = self.one_eic.network_id
        server_segment_nbr = 4
        eic = self.conn.connectivity.create_eic(self.one_meic.mcic_id,
                                                eic_name,
                                                network_id,
                                                server_segment_nbr)
        self.assertIsInstance(eic.cic_id, six.string_types)

    def test_delete_eic(self):
        eic_id = self.one_eic.cic_id
        meic_id = self.one_meic.mcic_id
        eic = self.conn.connectivity.delete_eic(meic_id, eic_id)
        # print eic.to_dict()
        self.assertEqual(eic_id, eic.id)

    def test_update_eic(self):
        eic_id = self.one_eic.cic_id
        meic_id = self.one_meic.mcic_id
        eic = self.conn.connectivity.update_eic(meic_id, eic_id, "xxx")
        # print eic.to_dict()
        self.assertEqual(eic_id, eic.id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from ecl.storage.v1 import volume_type as vt

IDENTIFIER = '6685584b-1eac-4da6-b5c3-555430cf68ff'
BASIC_EXAMPLE = {
    'id': IDENTIFIER,
    'name': 'piops_iscsi_na',
    'extra_specs': {
        'available_volume_size': [
            100, 250, 500, 1000, 2000, 4000, 8000, 12000
        ],
        'available_iops_per_gb': [
            '2', '4'
        ]
    }
}


class TestVolumeType(testtools.TestCase):

    def test_basic(self):
        sot = vt.VolumeType()
        self.assertEqual('volume_type', sot.resource_key)
        self.assertEqual('volume_types', sot.resources_key)
        self.assertEqual('/volume_types', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_get)
        self.assertEqual('storage', sot.service.service_type)

    def test_basic_detail(self):
        sot = vt.VolumeTypeDetail()
        self.assertEqual('volume_type', sot.resource_key)
        self.assertEqual('volume_types', sot.resources_key)
        self.assertEqual('/volume_types/detail', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_get)
        self.assertEqual('storage', sot.service.service_type)

    def test_make_basic(self):
        sot = vt.VolumeType(**BASIC_EXAMPLE)
        self.assertEqual(BASIC_EXAMPLE['id'], sot.id)
        self.assertEqual(BASIC_EXAMPLE['name'], sot.name)
        self.assertEqual(BASIC_EXAMPLE['extra_specs'], sot.extra_specs)



# -*- coding: utf-8 -*-

from ecl.connectivity import connectivity_service
from ecl import resource2


class Operation(resource2.Resource):

    resource_key = None
    resources_key = None

    base_path = '/operations'
    service = connectivity_service.ConnectivityService()

    # capabilities
    allow_get = True
    allow_list = True

    _query_mapping = resource2.QueryParameters("mcic_id", "cic_id")

    operation_id = resource2.Body('operation_id', alternate_id=True)
    operation_status = resource2.Body('operation_status')
    operation_type = resource2.Body('operation_type')
    operation_body = resource2.Body('operation_body', dict)
    mcic_id = resource2.Body('mcic_id')
    cic_id = resource2.Body('cic_id')
    user_id = resource2.Body('user_id')
    user_name = resource2.Body('user_name')
    receipt_date = resource2.Body('receipt_date')
    error_messages = resource2.Body('error_messages', list)



# -*- coding: utf-8 -*-

import base
from ecl import utils
from ecl import resource2
from ecl.network import network_service


class LoadBalancerSyslogServer(base.NetworkBaseResource):

    resources_key = "load_balancer_syslog_servers"
    resource_key = "load_balancer_syslog_server"
    service = network_service.NetworkService("v2.0")
    base_path = '/' + service.version + "/load_balancer_syslog_servers"

    _query_mapping = resource2.QueryParameters(
        "acl_logging", "appflow_logging", "date_format",
        "description", "id", "ip_address", "load_balancer_id",
        "log_facility", "log_level", "name", "port_number",
        "priority", "status", "tcp_logging", "tenant_id",
        "time_zone", "transport_type", "user_configurable_log_messages",
        "sort_key", "sort_dir",
    )
    # Capabilities
    allow_list = True
    allow_get = True
    allow_update = True
    allow_delete = True
    allow_create = True

    # Properties
    acl_logging = resource2.Body("acl_logging")

    appflow_logging = resource2.Body("appflow_logging")

    date_format = resource2.Body("date_format")

    #: Description of the Load Balancer Syslog Server.
    description = resource2.Body("description")

    #: Unique ID of the Load Balancer Syslog Server.
    id = resource2.Body("id")

    #: IP Address
    ip_address = resource2.Body("ip_address")

    #: The ID of load_balancer this load_balancer_interface belongs to.
    load_balancer_id = resource2.Body("load_balancer_id")

    log_facility = resource2.Body("log_facility")

    log_level = resource2.Body("log_level")

    #: Name of the Load Balancer
    name = resource2.Body("name")

    #: Port Number
    port_number = resource2.Body("port_number")

    #: Priority
    priority = resource2.Body("priority")

    #: The load_balancer Syslog Server status.
    status = resource2.Body("status")

    tcp_logging = resource2.Body("tcp_logging")

    #: Tenant ID of the owner (UUID)
    tenant_id = resource2.Body("tenant_id")

    #: Timezone
    time_zone = resource2.Body("time_zone")

    transport_type = resource2.Body("transport_type")

    user_configurable_log_messages = resource2.Body(
                    "user_configurable_log_messages")



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock
import testtools

from ecl.telemetry.v2 import sample

SAMPLE = {
    'sample_id': '0',
    'metadata': {'1': 'one'},
    'counter_name': '2',
    'project_id': '3',
    'recorded_at': '2015-03-09T12:15:57.233772',
    'resource_id': '5',
    'source': '6',
    'timestamp': '2015-03-09T12:15:57.233772',
    'type': '8',
    'unit': '9',
    'user_id': '10',
    'volume': '11.1',
}

OLD_SAMPLE = {
    'counter_name': '1',
    'counter_type': '2',
    'counter_unit': '3',
    'counter_volume': '4',
    'message_id': '0',
    'project_id': '5',
    'recorded_at': '2015-03-09T12:15:57.233772',
    'resource_id': '7',
    'resource_metadata': '8',
    'source': '9',
    'timestamp': '2015-03-09T12:15:57.233772',
    'user_id': '11',
}


class TestSample(testtools.TestCase):

    def test_basic(self):
        sot = sample.Sample(SAMPLE)
        self.assertIsNone(sot.resource_key)
        self.assertIsNone(sot.resources_key)
        self.assertEqual('/meters/%(counter_name)s', sot.base_path)
        self.assertEqual('metering', sot.service.service_type)
        self.assertTrue(sot.allow_create)
        self.assertFalse(sot.allow_retrieve)
        self.assertFalse(sot.allow_update)
        self.assertFalse(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_new(self):
        sot = sample.Sample(SAMPLE)
        self.assertEqual(SAMPLE['sample_id'], sot.id)
        self.assertEqual(SAMPLE['metadata'], sot.metadata)
        self.assertEqual(SAMPLE['counter_name'], sot.counter_name)
        self.assertEqual(SAMPLE['project_id'], sot.project_id)
        self.assertEqual(SAMPLE['recorded_at'], sot.recorded_at)
        self.assertEqual(SAMPLE['resource_id'], sot.resource_id)
        self.assertEqual(SAMPLE['source'], sot.source)
        self.assertEqual(SAMPLE['timestamp'], sot.generated_at)
        self.assertEqual(SAMPLE['type'], sot.type)
        self.assertEqual(SAMPLE['unit'], sot.unit)
        self.assertEqual(SAMPLE['user_id'], sot.user_id)
        self.assertEqual(SAMPLE['volume'], sot.volume)

    def test_make_old(self):
        sot = sample.Sample(OLD_SAMPLE)
        self.assertIsNone(sot.id)
        self.assertEqual(OLD_SAMPLE['counter_name'], sot.counter_name)
        self.assertEqual(OLD_SAMPLE['counter_type'], sot.type)
        self.assertEqual(OLD_SAMPLE['counter_unit'], sot.unit)
        self.assertEqual(OLD_SAMPLE['counter_volume'], sot.volume)
        self.assertEqual(OLD_SAMPLE['project_id'], sot.project_id)
        self.assertEqual(OLD_SAMPLE['recorded_at'], sot.recorded_at)
        self.assertEqual(OLD_SAMPLE['resource_id'], sot.resource_id)
        self.assertEqual(OLD_SAMPLE['resource_metadata'], sot.metadata)
        self.assertEqual(OLD_SAMPLE['source'], sot.source)
        self.assertEqual(OLD_SAMPLE['timestamp'], sot.generated_at)
        self.assertEqual(OLD_SAMPLE['user_id'], sot.user_id)

    def test_list(self):
        sess = mock.Mock()
        resp = mock.Mock()
        resp.json = mock.Mock(return_value=[SAMPLE, OLD_SAMPLE])
        sess.get = mock.Mock(return_value=resp)
        path_args = {'counter_name': 'name_of_meter'}

        found = sample.Sample.list(sess, path_args=path_args)
        first = next(found)
        self.assertEqual(SAMPLE['sample_id'], first.id)
        self.assertEqual(SAMPLE['metadata'], first.metadata)
        self.assertEqual(SAMPLE['counter_name'], first.counter_name)
        self.assertEqual(SAMPLE['project_id'], first.project_id)
        self.assertEqual(SAMPLE['recorded_at'], first.recorded_at)
        self.assertEqual(SAMPLE['resource_id'], first.resource_id)
        self.assertEqual(SAMPLE['source'], first.source)
        self.assertEqual(SAMPLE['timestamp'], first.generated_at)
        self.assertEqual(SAMPLE['type'], first.type)
        self.assertEqual(SAMPLE['unit'], first.unit)
        self.assertEqual(SAMPLE['user_id'], first.user_id)
        self.assertEqual(SAMPLE['volume'], first.volume)
        second = next(found)
        self.assertEqual(OLD_SAMPLE['counter_name'], second.counter_name)

    def test_create(self):
        sess = mock.Mock()
        resp = mock.Mock()
        resp.json = mock.Mock(return_value=[SAMPLE])
        sess.post = mock.Mock(return_value=resp)

        data = {'sample_id': None,
                'counter_name': 'temperature',
                'project_id': 'project',
                'resource_id': 'resource',
                'type': 'gauge',
                'unit': 'instance',
                'volume': '98.6'}
        new_sample = sample.Sample.new(**data)

        new_sample.create(sess)
        url = '/meters/temperature'
        sess.post.assert_called_with(url, endpoint_filter=new_sample.service,
                                     json=[data])
        self.assertIsNone(new_sample.id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.sss import  sss_service
from ecl import resource2


class Contract(resource2.Resource):
    resource_key = None
    resources_key = None
    base_path = '/contracts'
    service = sss_service.SssService()

    # Capabilities
    allow_create = True
    allow_get = True
    allow_delete = True
    allow_list = True
    put_create = False

    # Properties
    #: Contract ID.
    contract_id = resource2.Body('contract_id', alternate_id=True)
    #: Login ID of user.
    login_id = resource2.Body('login_id')
    #: E-mail address of the user.
    mail_address = resource2.Body('mail_addresss')
    #: Password of the user. If user set this item as blank, the system
    #: set initial random password automatically.
    password = resource2.Body('password')
    #: By using this item, the partner API user can associate optional
    #: string to the constract(e.g. The end user management ID in the
    #: partner user's system). Note that this ID will be NOT used to
    #: control the contract in ECL2.0 internal system. If the item is
    #: set as blank, ECL 2.0 system set the end user's contract ID
    #: automatically
    external_reference_id = resource2.Body('external_reference_id')
    #: Setting true or false(default is false). This item designate
    #: whether the system should send to the login_ID and Password with
    #: e-mail.
    notify_password = resource2.Body('notify_password')
    #: User id. format is ecid[0-9]{10}.
    user_id = resource2.Body('user_id')
    #: This user's API key for keystone authentication.
    keystone_name = resource2.Body('keystone_name')
    #: This user's API secret for keystone authentication.
    keystone_password = resource2.Body('keystone_password')
    #: Keystone address this user can use to get token for SSS API
    #: request.
    keystone_endpoint = resource2.Body('keystone_endpoint')
    #: SSS endpoint recommended for user.
    sss_endpoint = resource2.Body('sss_endpoint')
    #: status
    status = resource2.Body('status')
    #: customer_name
    customer_name = resource2.Body('customer_name')
    #: This ID is equal to external_reference_id.
    customer_number = resource2.Body('customer_number')
    #: Channel name.
    channel_name = resource2.Body('channel_name')
    #: Channel ID of this contract.
    channel_id = resource2.Body('channel_id')
    #: Whether this channel_id is the partner management channel(true),
    #: or not(false).
    management_channel = resource2.Body('management_channel')
    #: The owner user ID in the Contract
    contract_owner_user_id = resource2.Body('contract_owner_user_id')
    #: Start time of the contract
    start_time = resource2.Body('start_time')
    #: End time of the contract
    end_time = resource2.Body('end_time')
    #: Internal use(Login type)
    login_integration = resource2.Body('login_integration')
    #: Access point to use APIs in the decided region
    sss_endpoint = resource2.Body('sss_endpoint')
    #: keystone_endpoint
    keystone_endpoint = resource2.Body('keystone_endpoint')
    #: Internal use
    company_code = resource2.Body('company_code')
    #: Internal use
    glass_customer_id = resource2.Body('glass_customer_id')
    #: Internal use
    glass_user_id = resource2.Body('glass_user_id')
    #: The status of the acquired charge information (estimated:
    #: undetermined / fixed: Committed)
    charge_status = resource2.Body('charge_status')
    #: Period of the acquired charge information(Start time)
    cycle_start_time = resource2.Body('cycle_start_time')
    #: Period of the acquired charge information(End time)
    cycle_end_time = resource2.Body('cycle_end_time')
    #: The detail of billing information
    charge_data = resource2.Body('charge_data')
    #: The designated contract ID by request's target_contract_id.
    owner_contract_id = resource2.Body('owner_contract_id')

    def list(self, session, channel_id, include_deleted):
        """List contracts by channel id"""

        url = self.base_path + '?channel_id=%s&include_deleted=%s' % (
            channel_id, include_deleted
        )
        resp = session.get(
            url, endpoint_filter=self.service
        )
        self._translate_response(resp, has_body=True)
        return self

    def get_billing_info(self, session, channel_id, target_month):
        """Get billing information by channel id and target month"""

        url = self.base_path + '/%s/billing/%s' % (
            channel_id, target_month)
        resp = session.get(
            url, endpoint_filter=self.service
        )
        self._translate_response(resp, has_body=True)
        return self

    def get_monthly_billing_of_each_contract(
            self, session, contract_id, target_month, target_contract_id
    ):
        """Get montly billing for each contract."""

        url = self.base_path + "/%s/billing/%s/target_contract/%s" % (
            contract_id, target_month, target_contract_id
        )
        resp = session.get(url, endpoint_filter=self.service)
        self._translate_response(resp, has_body=True)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl import service_filter


class BaremetalService(service_filter.ServiceFilter):
    """Baremetal service."""

    valid_versions = [service_filter.ValidVersion('v2')]

    def __init__(self, version=None):
        """Create a baremetal service."""
        super(BaremetalService, self).__init__(
            service_type='baremetal-server',
            version=version
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl import proxy
from ecl.telemetry.v2 import alarm as _alarm
from ecl.telemetry.v2 import alarm_change as _alarm_change
from ecl.telemetry.v2 import capability
from ecl.telemetry.v2 import meter as _meter
from ecl.telemetry.v2 import resource as _resource
from ecl.telemetry.v2 import sample
from ecl.telemetry.v2 import statistics


class Proxy(proxy.BaseProxy):
    """.. caution:: This API is a work in progress and is subject to change."""

    def create_alarm(self, **attrs):
        """Create a new alarm from attributes

        :param dict attrs: Keyword arguments which will be used to create
                           a :class:`~ecl.telemetry.v2.alarm.Alarm`,
                           comprised of the properties on the Alarm class.

        :returns: The results of alarm creation
        :rtype: :class:`~ecl.telemetry.v2.alarm.Alarm`
        """
        return self._create(_alarm.Alarm, **attrs)

    def delete_alarm(self, alarm, ignore_missing=False):
        """Delete an alarm

        :param alarm: The value can be either the ID of an alarm or a
                      :class:`~ecl.telemetry.v2.alarm.Alarm` instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the alarm does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent alarm.

        :returns: ``None``
        """
        self._delete(_alarm.Alarm, alarm, ignore_missing=ignore_missing)

    def find_alarm(self, name_or_id, ignore_missing=False):
        """Find a single alarm

        :param name_or_id: The name or ID of a alarm.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.telemetry.v2.alarm.Alarm` or None
        """
        return self._find(_alarm.Alarm, name_or_id,
                          ignore_missing=ignore_missing)

    def get_alarm(self, alarm):
        """Get a single alarm

        :param alarm: The value can be the ID of an alarm or a
                      :class:`~ecl.telemetry.v2.alarm.Alarm` instance.

        :returns: One :class:`~ecl.telemetry.v2.alarm.Alarm`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_alarm.Alarm, alarm)

    def alarms(self, **query):
        """Return a generator of alarms

        :param kwargs \*\*query: Optional query parameters to be sent to limit
                                 the resources being returned.

        :returns: A generator of alarm objects
        :rtype: :class:`~ecl.telemetry.v2.alarm.Alarm`
        """
        return self._list(_alarm.Alarm, paginated=False, **query)

    def update_alarm(self, alarm, **attrs):
        """Update a alarm

        :param alarm: Either the id of a alarm or a
                      :class:`~ecl.telemetry.v2.alarm.Alarm` instance.
        :attrs kwargs: The attributes to update on the alarm represented
                       by ``value``.

        :returns: The updated alarm
        :rtype: :class:`~ecl.telemetry.v2.alarm.Alarm`
        """
        return self._update(_alarm.Alarm, alarm, **attrs)

    def find_alarm_change(self, name_or_id, ignore_missing=False):
        """Find a single alarm change

        :param name_or_id: The name or ID of a alarm change.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.telemetry.v2.alarm_change.AlarmChange`
                  or None
        """
        return self._find(_alarm_change.AlarmChange, name_or_id,
                          ignore_missing=ignore_missing)

    def alarm_changes(self, alarm, **query):
        """Return a generator of alarm changes

        :param alarm: Alarm resource or id for alarm.
        :param kwargs \*\*query: Optional query parameters to be sent to limit
                                 the resources being returned.

        :returns: A generator of alarm change objects
        :rtype: :class:`~ecl.telemetry.v2.alarm_change.AlarmChange`
        """
        alarm_id = _alarm.Alarm.from_id(alarm).id
        return self._list(_alarm_change.AlarmChange, paginated=False,
                          path_args={'alarm_id': alarm_id}, **query)

    def find_capability(self, name_or_id, ignore_missing=False):
        """Find a single capability

        :param name_or_id: The name or ID of a capability.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.telemetry.v2.capability.Capability`
                  or None
        """
        return self._find(capability.Capability, name_or_id,
                          ignore_missing=ignore_missing)

    def capabilities(self, **query):
        """Return a generator of capabilities

        :param kwargs \*\*query: Optional query parameters to be sent to limit
                                 the resources being returned.

        :returns: A generator of capability objects
        :rtype: :class:`~ecl.telemetry.v2.capability.Capability`
        """
        return self._list(capability.Capability, paginated=False, **query)

    def find_meter(self, name_or_id, ignore_missing=False):
        """Find a single meter

        :param name_or_id: The name or ID of a meter.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.telemetry.v2.meter.Meter` or None
        """
        return self._find(_meter.Meter, name_or_id,
                          ignore_missing=ignore_missing)

    def meters(self, **query):
        """Return a generator of meters

        :param kwargs \*\*query: Optional query parameters to be sent to limit
                                 the resources being returned.

        :returns: A generator of meter objects
        :rtype: :class:`~ecl.telemetry.v2.meter.Meter`
        """
        return self._list(_meter.Meter, paginated=False, **query)

    def find_resource(self, name_or_id, ignore_missing=False):
        """Find a single resource

        :param name_or_id: The name or ID of a resource.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.telemetry.v2.resource.Resource` or
                  None
        """
        return self._find(_resource.Resource, name_or_id,
                          ignore_missing=ignore_missing)

    def get_resource(self, resource):
        """Get a single resource

        :param resource: The value can be the ID of a resource or a
                         :class:`~ecl.telemetry.v2.resource.Resource`
                         instance.

        :returns: One :class:`~ecl.telemetry.v2.resource.Resource`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_resource.Resource, resource)

    def resources(self, **query):
        """Return a generator of resources

        :param kwargs \*\*query: Optional query parameters to be sent to limit
                                 the resources being returned.

        :returns: A generator of resource objects
        :rtype: :class:`~ecl.telemetry.v2.resource.Resource`
        """
        return self._list(_resource.Resource, paginated=False, **query)

    def create_sample(self, **attrs):
        """Create a new sample from attributes

        :param dict attrs: Keyword arguments which will be used to create
                           a :class:`~ecl.telemetry.v2.sample.Sample`,
                           comprised of the properties on the Sample class.

        :returns: The results of sample creation
        :rtype: :class:`~ecl.telemetry.v2.sample.Sample`
        """
        return self._create(sample.Sample, **attrs)

    def find_sample(self, name_or_id, ignore_missing=False):
        """Find a single sample

        :param name_or_id: The name or ID of a sample.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.telemetry.v2.sample.Sample` or None
        """
        return self._find(sample.Sample, name_or_id,
                          ignore_missing=ignore_missing)

    def samples(self, meter, **query):
        """Return a generator of samples

        :param value: Meter resource or name for a meter.
        :param kwargs \*\*query: Optional query parameters to be sent to limit
                                 the resources being returned.

        :returns: A generator of sample objects
        :rtype: :class:`~ecl.telemetry.v2.sample.Sample`
        """
        meter_name = _meter.Meter.from_name(meter).name
        return self._list(sample.Sample, paginated=False,
                          path_args={'counter_name': meter_name}, **query)

    def find_statistics(self, name_or_id, ignore_missing=False):
        """Find a single statistics

        :param name_or_id: The name or ID of a statistics.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.telemetry.v2.statistics.Statistics`
                  or None
        """
        return self._find(statistics.Statistics, name_or_id,
                          ignore_missing=ignore_missing)

    def statistics(self, meter, **query):
        """Return a generator of statistics

        :param meter: Meter resource or name for a meter.
        :param kwargs \*\*query: Optional query parameters to be sent to limit
                                 the resources being returned.

        :returns: A generator of statistics objects
        :rtype: :class:`~ecl.telemetry.v2.statistics.Statistics`
        """
        meter_name = _meter.Meter.from_name(meter).name
        return self._list(statistics.Statistics, paginated=False,
                          path_args={'meter_name': meter_name}, **query)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six
from ecl.tests.functional import base

class TestNicPorts(base.BaseFunctionalTest):

    @classmethod
    def test_01_list(cls):
        server_id = "752aac2e-4b82-4d47-a7c7-fcbd0cbc86e2"
        ports = list(cls.conn.baremetal.nic_physical_ports(server_id))
        cls.port = ports[0]
        assert isinstance(cls.port.id, six.string_types)
        assert isinstance(cls.port.mac_addr, six.string_types)
        assert isinstance(cls.port.network_physical_port_id,
                          six.string_types)
        assert isinstance(cls.port.plane, six.string_types)
        assert isinstance(cls.port.attached_ports, list)
        assert isinstance(cls.port.hardware_id, six.string_types)

    def test_02_show(self):
        server_id = "752aac2e-4b82-4d47-a7c7-fcbd0cbc86e2"
        port = "e0ca770c-2ced-4099-b051-26512d9af43d"
        port = self.conn.baremetal.get_nic_physical_port(server_id, port)
        self.assertIsInstance(port.hardware_id, six.string_types)
        self.assertIsInstance(port.id, six.string_types)
        self.assertIsInstance(port.mac_addr, six.string_types)
        self.assertIsInstance(port.network_physical_port_id,
                              six.string_types)
        self.assertIsInstance(port.attached_ports, list)
        self.assertIsInstance(port.plane, six.string_types)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock

from ecl.tests.unit import base


class TestProxyBase(base.TestCase):
    def setUp(self):
        super(TestProxyBase, self).setUp()
        self.session = mock.Mock()

    def _add_path_args_for_verify(self, path_args, method_args,
                                  expected_kwargs, value=None):
        if path_args is not None:
            if value is None:
                for key in path_args:
                    method_args.append(path_args[key])
            expected_kwargs['path_args'] = path_args

    def _verify(self, mock_method, test_method,
                method_args=None, method_kwargs=None,
                expected_args=None, expected_kwargs=None,
                expected_result=None):
        with mock.patch(mock_method) as mocked:
            mocked.return_value = expected_result
            if any([method_args, method_kwargs,
                    expected_args, expected_kwargs]):
                method_args = method_args or ()
                method_kwargs = method_kwargs or {}
                expected_args = expected_args or ()
                expected_kwargs = expected_kwargs or {}

                self.assertEqual(expected_result, test_method(*method_args,
                                 **method_kwargs))
                mocked.assert_called_with(self.session,
                                          *expected_args, **expected_kwargs)
            else:
                self.assertEqual(expected_result, test_method())
                mocked.assert_called_with(self.session)

    # NOTE(briancurtin): This is a duplicate version of _verify that is
    # temporarily here while we shift APIs. The difference is that
    # calls from the Proxy classes aren't going to be going directly into
    # the Resource layer anymore, so they don't pass in the session which
    # was tested in assert_called_with.
    # This is being done in lieu of adding logic and complicating
    # the _verify method. It will be removed once there is one API to
    # be verifying.
    def _verify2(self, mock_method, test_method,
                 method_args=None, method_kwargs=None, method_result=None,
                 expected_args=None, expected_kwargs=None,
                 expected_result=None):
        with mock.patch(mock_method) as mocked:
            mocked.return_value = expected_result
            if any([method_args, method_kwargs,
                    expected_args, expected_kwargs]):
                method_args = method_args or ()
                method_kwargs = method_kwargs or {}
                expected_args = expected_args or ()
                expected_kwargs = expected_kwargs or {}

                if method_result:
                    self.assertEqual(method_result, test_method(*method_args,
                                     **method_kwargs))
                else:
                    self.assertEqual(expected_result, test_method(*method_args,
                                     **method_kwargs))
                mocked.assert_called_with(*expected_args, **expected_kwargs)
            else:
                self.assertEqual(expected_result, test_method())
                mocked.assert_called_with(self.session)

    def verify_create(self, test_method, resource_type,
                      mock_method="ecl.proxy2.BaseProxy._create",
                      expected_result="result", **kwargs):
        the_kwargs = {"x": 1, "y": 2, "z": 3}
        method_kwargs = kwargs.pop("method_kwargs", the_kwargs)
        expected_args = [resource_type]
        expected_kwargs = kwargs.pop("expected_kwargs", the_kwargs)

        self._verify2(mock_method, test_method,
                      expected_result=expected_result,
                      method_kwargs=method_kwargs,
                      expected_args=expected_args,
                      expected_kwargs=expected_kwargs,
                      **kwargs)

    def verify_delete(self, test_method, resource_type, ignore,
                      input_path_args=None, expected_path_args=None,
                      method_kwargs=None, expected_args=None,
                      expected_kwargs=None,
                      mock_method="ecl.proxy2.BaseProxy._delete"):
        method_args = ["resource_or_id"]
        method_kwargs = method_kwargs or {}
        method_kwargs["ignore_missing"] = ignore
        if isinstance(input_path_args, dict):
            for key in input_path_args:
                method_kwargs[key] = input_path_args[key]
        elif isinstance(input_path_args, list):
            method_args = input_path_args
        expected_kwargs = expected_kwargs or {}
        expected_kwargs["ignore_missing"] = ignore
        if expected_path_args:
            expected_kwargs.update(expected_path_args)
        expected_args = expected_args or [resource_type, "resource_or_id"]
        self._verify2(mock_method, test_method,
                      method_args=method_args,
                      method_kwargs=method_kwargs,
                      expected_args=expected_args,
                      expected_kwargs=expected_kwargs)

    def verify_get(self, test_method, resource_type, value=None, args=None,
                   mock_method="ecl.proxy2.BaseProxy._get",
                   ignore_value=False, **kwargs):
        the_value = value
        if value is None:
            the_value = [] if ignore_value else ["value"]
        expected_args = kwargs.pop("expected_args", [])
        expected_kwargs = kwargs.pop("expected_kwargs", {})
        method_kwargs = kwargs.pop("method_kwargs", kwargs)
        if args:
            expected_kwargs["args"] = args
        if kwargs:
            expected_kwargs["path_args"] = kwargs
        if not expected_args:
            expected_args = [resource_type] + the_value
        self._verify2(mock_method, test_method,
                      method_args=the_value,
                      method_kwargs=method_kwargs or {},
                      expected_args=expected_args,
                      expected_kwargs=expected_kwargs)

    def verify_head(self, test_method, resource_type,
                    mock_method="ecl.proxy2.BaseProxy._head",
                    value=None, **kwargs):
        the_value = [value] if value is not None else []
        expected_kwargs = {"path_args": kwargs} if kwargs else {}
        self._verify2(mock_method, test_method,
                      method_args=the_value,
                      method_kwargs=kwargs,
                      expected_args=[resource_type] + the_value,
                      expected_kwargs=expected_kwargs)

    def verify_find(self, test_method, resource_type, value=None,
                    mock_method="ecl.proxy2.BaseProxy._find",
                    path_args=None, **kwargs):
        method_args = value or ["name_or_id"]
        expected_kwargs = {}

        self._add_path_args_for_verify(path_args, method_args, expected_kwargs,
                                       value=value)

        # TODO(briancurtin): if sub-tests worked in this mess of
        # test dependencies, the following would be a lot easier to work with.
        expected_kwargs["ignore_missing"] = False
        self._verify2(mock_method, test_method,
                      method_args=method_args + [False],
                      expected_args=[resource_type, "name_or_id"],
                      expected_kwargs=expected_kwargs,
                      expected_result="result",
                      **kwargs)

        expected_kwargs["ignore_missing"] = True
        self._verify2(mock_method, test_method,
                      method_args=method_args + [True],
                      expected_args=[resource_type, "name_or_id"],
                      expected_kwargs=expected_kwargs,
                      expected_result="result",
                      **kwargs)

    def verify_list(self, test_method, resource_type, paginated=False,
                    mock_method="ecl.proxy2.BaseProxy._list",
                    **kwargs):
        expected_kwargs = kwargs.pop("expected_kwargs", {})
        expected_kwargs.update({"paginated": paginated})
        method_kwargs = kwargs.pop("method_kwargs", {})
        self._verify2(mock_method, test_method,
                      method_kwargs=method_kwargs,
                      expected_args=[resource_type],
                      expected_kwargs=expected_kwargs,
                      expected_result=["result"],
                      **kwargs)

    def verify_list_no_kwargs(self, test_method, resource_type,
                              paginated=False,
                              mock_method="ecl.proxy2.BaseProxy._list"):
        self._verify2(mock_method, test_method,
                      method_kwargs={},
                      expected_args=[resource_type],
                      expected_kwargs={"paginated": paginated},
                      expected_result=["result"])

    def verify_update(self, test_method, resource_type, value=None,
                      mock_method="ecl.proxy2.BaseProxy._update",
                      expected_result="result", path_args=None, **kwargs):
        method_args = value or ["resource_or_id"]
        method_kwargs = {"x": 1, "y": 2, "z": 3}
        expected_args = kwargs.pop("expected_args", ["resource_or_id"])
        expected_kwargs = method_kwargs.copy()

        self._add_path_args_for_verify(path_args, method_args, expected_kwargs,
                                       value=value)

        self._verify2(mock_method, test_method,
                      expected_result=expected_result,
                      method_args=method_args,
                      method_kwargs=method_kwargs,
                      expected_args=[resource_type] + expected_args,
                      expected_kwargs=expected_kwargs,
                      **kwargs)

    def verify_wait_for_status(
            self, test_method,
            mock_method="ecl.resource2.wait_for_status", **kwargs):
        self._verify(mock_method, test_method, **kwargs)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from ecl.orchestration.v1 import software_config


FAKE_ID = 'ce8ae86c-9810-4cb1-8888-7fb53bc523bf'
FAKE_NAME = 'test_software_config'
FAKE = {
    'id': FAKE_ID,
    'name': FAKE_NAME,
    'config': 'fake config',
    'creation_time': '2015-03-09T12:15:57',
    'group': 'fake group',
    'inputs': [{'foo': 'bar'}],
    'outputs': [{'baz': 'zoo'}],
    'options': {'key': 'value'},
}


class TestSoftwareConfig(testtools.TestCase):

    def test_basic(self):
        sot = software_config.SoftwareConfig()
        self.assertEqual('software_config', sot.resource_key)
        self.assertEqual('software_configs', sot.resources_key)
        self.assertEqual('/software_configs', sot.base_path)
        self.assertEqual('orchestration', sot.service.service_type)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_get)
        self.assertFalse(sot.allow_update)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = software_config.SoftwareConfig(**FAKE)
        self.assertEqual(FAKE_ID, sot.id)
        self.assertEqual(FAKE_NAME, sot.name)
        self.assertEqual(FAKE['config'], sot.config)
        self.assertEqual(FAKE['creation_time'], sot.created_at)
        self.assertEqual(FAKE['group'], sot.group)
        self.assertEqual(FAKE['inputs'], sot.inputs)
        self.assertEqual(FAKE['outputs'], sot.outputs)
        self.assertEqual(FAKE['options'], sot.options)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock

from ecl import exceptions
from ecl.image.v2 import _proxy
from ecl.image.v2 import image
from ecl.image.v2 import member
from ecl.tests.unit import test_proxy_base2


class TestImageProxy(test_proxy_base2.TestProxyBase):
    def setUp(self):
        super(TestImageProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)

    def test_image_create_no_args(self):
        # container_format and disk_format are required args
        self.assertRaises(exceptions.InvalidRequest, self.proxy.upload_image)

    def test_image_create(self):
        # NOTE: This doesn't use any of the base class verify methods
        # because it ends up making two separate calls to complete the
        # operation.
        created_image = mock.Mock(spec=image.Image(id="id"))

        self.proxy._create = mock.Mock()
        self.proxy._create.return_value = created_image

        rv = self.proxy.upload_image(data="data", container_format="x",
                                     disk_format="y", name="z")

        self.proxy._create.assert_called_with(image.Image,
                                              container_format="x",
                                              disk_format="y",
                                              name="z")
        created_image.upload.assert_called_with(self.session)
        self.assertEqual(rv, created_image)

    def test_image_delete(self):
        self.verify_delete(self.proxy.delete_image, image.Image, False)

    def test_image_delete_ignore(self):
        self.verify_delete(self.proxy.delete_image, image.Image, True)

    def test_image_update(self):
        self.verify_update(self.proxy.update_image, image.Image)

    def test_image_get(self):
        self.verify_get(self.proxy.get_image, image.Image)

    def test_images(self):
        self.verify_list(list(self.proxy.images), image.Image, paginated=True)

    def test_add_tag(self):
        self._verify("ecl.image.v2.image.Image.add_tag",
                     self.proxy.add_tag,
                     method_args=["image", "tag"],
                     expected_args=["tag"])

    def test_remove_tag(self):
        self._verify("ecl.image.v2.image.Image.remove_tag",
                     self.proxy.remove_tag,
                     method_args=["image", "tag"],
                     expected_args=["tag"])

    def test_member_update(self):
        self._verify2("ecl.proxy2.BaseProxy._update",
                      self.proxy.update_member,
                      method_args=['member_id', 'image_id', 'status'],
                      expected_args=[member.Member],
                      expected_kwargs={'member_id': 'member_id',
                                       'image_id': 'image_id'})

    def test_member_get(self):
        self._verify2("ecl.proxy2.BaseProxy._get",
                      self.proxy.get_member,
                      method_args=['image_id','member'],
                      expected_args=[member.Member, 'member'],
                      expected_kwargs={"image_id":"image_id"})

    def test_members(self):
        self.verify_list(self.proxy.members, member.Member, paginated=False,
                         method_args=('image_1',),
                         expected_kwargs={'image_id': 'image_1'})



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six
from ecl.tests.functional import base


class TestUser(base.BaseFunctionalTest):

    @classmethod
    def setUpClass(cls):
        super(TestUser, cls).setUpClass()
        users_list = list(cls.conn.sss.users())
        cls.one_user = users_list[0]

    def test_users(self):
        users = list(self.conn.sss.users())
        for i in users:
            print i.id + " " + i.login_id
        self.assertGreater(len(users), 0)

    def test_01_find_user_by_id(self):
        user = self.conn.sss.find_user(self.one_user.id)
        self.assertIsInstance(user.login_id, six.string_types)
        self.assertIsInstance(user.mail_address, six.string_types)
        self.assertIsInstance(user.contract_owner, bool)
        self.assertIsInstance(user.keystone_name, six.string_types)
        self.assertIsInstance(user.keystone_password, six.string_types)
        self.assertIsInstance(user.keystone_endpoint, six.string_types)
        self.assertIsInstance(user.sss_endpoint, six.string_types)
        self.assertIsInstance(user.contract_id, six.string_types)
        self.assertIsInstance(user.login_integration, six.string_types)
        self.assertIsInstance(user.external_reference_id, six.string_types)
        self.assertEqual(user.id, self.one_user.id)

    def test_02_get_user_by_id(self):
        user = self.conn.sss.get_user(self.one_user.id)
        self.assertIsInstance(user.login_id, six.string_types)
        self.assertIsInstance(user.mail_address, six.string_types)
        self.assertIsInstance(user.contract_owner, bool)
        self.assertIsInstance(user.keystone_name, six.string_types)
        self.assertIsInstance(user.keystone_password, six.string_types)
        self.assertIsInstance(user.keystone_endpoint, six.string_types)
        self.assertIsInstance(user.sss_endpoint, six.string_types)
        self.assertIsInstance(user.contract_id, six.string_types)
        self.assertIsInstance(user.login_integration, six.string_types)
        self.assertIsInstance(user.external_reference_id, six.string_types)
        self.assertEqual(user.id, self.one_user.id)

    @classmethod
    def test_03_create_user(cls):
        user = cls.conn.sss.create_user(
            login_id="sdk_test02",
            mail_address="example@ntt.com",
            password="Mypassword01",
            notify_password="false",
        )
        assert isinstance(user.login_id, six.string_types)
        assert isinstance(user.mail_address, six.string_types)
        assert isinstance(user.id, six.string_types)
        assert isinstance(user.keystone_name, six.string_types)
        assert isinstance(user.keystone_password, six.string_types)
        assert isinstance(user.keystone_endpoint, six.string_types)
        assert isinstance(user.sss_endpoint, six.string_types)
        assert isinstance(user.password, six.string_types)
        cls.new_account_id = user.id

    def test_04_update_user(self):
        self.conn.sss.update_user(
            self.new_account_id,
            mail_address="example2@ntt.com",
        )
        user = self.conn.sss.get_user(self.new_account_id)
        self.assertEqual(user.mail_address, "example2@ntt.com")

    def test_05_delete_user(self):
        self.conn.sss.delete_user(self.new_account_id)




# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import  testtools
from ecl.sss.v1 import channel

BASIC_EXAMPLE = {
    "channel_id": "Chiquia",
    "channel_name": "cheroot",
    "language": "ha",
    "management_channel": True,
    "parent_channel_id": None,
    "contracts": [
        {
            "contract_id": "excusableness",
            "status": "enable"
        }
    ]
}

class TestRole(testtools.TestCase):

    def test_role(self):
        sot = channel.Channel()
        self.assertEqual("channels", sot.resources_key)
        self.assertEqual(None, sot.resource_key)
        self.assertEqual("/channels?get_contracts=%(get_contracts)s", sot.base_path)
        self.assertEqual('sss', sot.service.service_type)

        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_get)
        self.assertFalse(sot.allow_update)
        self.assertFalse(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_role(self):
        sot = channel.Channel(**BASIC_EXAMPLE)
        self.assertEqual(BASIC_EXAMPLE['channel_id'], sot.channel_id)
        self.assertEqual(BASIC_EXAMPLE['channel_name'], sot.channel_name)
        self.assertEqual(BASIC_EXAMPLE['language'], sot.language)
        self.assertEqual(BASIC_EXAMPLE['management_channel'], sot.management_channel)
        self.assertEqual(BASIC_EXAMPLE['parent_channel_id'], sot.parent_channel_id)
        self.assertEqual(BASIC_EXAMPLE['contracts'], sot.contracts)



# -*- coding: utf-8 -*-

from ecl.block_store import block_store_service
from ecl import resource


class Quota(resource.Resource):
    resource_key = "quota_set"
    resources_key = "quota_sets"
    base_path = '/os-quota-sets'
    service = block_store_service.BlockStoreService()

    # capabilities
    allow_retrieve = True

    id = resource.prop("id")


class DetailQuota(resource.Resource):
    resource_key = "quota_set"
    resources_key = "quota_sets"
    base_path = '/os-quota-sets/%(tenant_id)s/detail'

    service = block_store_service.BlockStoreService()

    # capabilities
    allow_retrieve = True

    id = resource.prop("id")


class DefaultQuota(resource.Resource):
    resource_key = "quota_set"
    resources_key = "quota_sets"
    base_path = '/os-quota-sets/%(tenant_id)s/defaults'

    service = block_store_service.BlockStoreService()

    allow_retrieve = True

    id = resource.prop("id")



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.compute.v2 import availability_zone as _availability_zone
from ecl.compute.v2 import extension
from ecl.compute.v2 import flavor as _flavor
from ecl.compute.v2 import image as _image
from ecl.compute.v2 import keypair as _keypair
from ecl.compute.v2 import limits
from ecl.compute.v2 import server as _server
from ecl.compute.v2 import server_action as _server_action
from ecl.compute.v2 import server_interface as _server_interface
from ecl.compute.v2 import server_volume as _server_volume
from ecl.compute.v2 import quota as _quota
from ecl.compute.v2 import volume as _volume
from ecl.image.v2 import image

from ecl import proxy2
from ecl import resource2


class Proxy(proxy2.BaseProxy):

    def servers(self, details=True, **query):
        """Retrieve a list of servers

        :param bool details: When set to ``False``
                    :class:`~ecl.compute.v2.server.Server` instances
                    will be returned. The default, ``True``, will cause
                    :class:`~ecl.compute.v2.server.ServerDetail`
                    instances to be returned.
        :param kwargs \*\*query: Optional query parameters to be sent to limit
            the servers being returned.  Available parameters include:

            * changes_since: A time/date stamp for when the server last changed status.
            * image: An image resource or ID.
            * flavor: A flavor resource or ID.
            * name: Name of the server as a string.
            * status: Value of the status of the server so that you can filter on "ACTIVE" for example.
            * host: Name of the host as a string.
            * limit: Requests a specified page size of returned items from the query.
            * marker: Specifies the ID of the last-seen item.
        :returns: A list of :class:`~ecl.compute.v2.server.Server`
        """
        srv = _server.ServerDetail if details else _server.Server
        return list(self._list(srv, paginated=True, **query))

    def create_server(self, flavor_id, name, disk_config=None, image_id=None,
                      min_count=None, max_count=None, availability_zone=None,
                      config_drive=None, key_name=None, user_data=None,
                      block_device_mapping=None, block_device_mapping_v2=None,
                      metadata=None, networks=None, personality=None, admin_pass=None, **attrs):
        """Create a new server from attributes

        :param string flavor_id: ID of server
        :param string name: Name of server
        :param dict disk_config: a single partition which is expanded
                to the size of the flavor selected
        :param string image_id: if block_device_mapping_v2 is not specified,
                it is require
        :param int min_count: minim count of instance
        :param int max_count: maxmum count of instance
        :param string availability_zone: availability zone
        :param boolean config_drive: Enables metadata injection in a server
                through a configuration drive
        :param string key_name: key name
        :param string (Base64 encoded) user_data: user data
        :param array block_device_mapping: block device mapping info
        :param array block_device_mapping_v2:  block device mapping v2 info
        :param dict metadata: metadata of the server
        :param array networks: if a tenant has more than two networks, it is required
        :param array personality: This param will not run for ECL2.0
        :param string admin_pass: the administrator password for the server
        :param kwargs attrs: Keyword arguments which will be used to create
            a :class:`~ecl.compute.v2.server.Server`,
            comprised of the properties on the Server class.
        :returns: :class:`~ecl.compute.v2.server.Server`
        """
        attrs.update({"flavor_id": flavor_id})
        attrs.update({"name": name})
        if disk_config is not None:
            attrs.update({"disk_config": disk_config})
        if image_id is not None:
            attrs.update({"image_id": image_id})
        if min_count:
            attrs.update({"min_count": min_count})
        if max_count:
            attrs.update({"max_count": max_count})
        if availability_zone:
            attrs.update({"availability_zone": availability_zone})
        if config_drive:
            attrs.update({"config_drive": config_drive})
        if key_name:
            attrs.update({"key_name": key_name})
        if user_data:
            attrs.update({"user_data": user_data})
        if block_device_mapping:
            attrs.update({"block_device_mapping": block_device_mapping})
        if block_device_mapping_v2:
            attrs.update({"block_device_mapping_v2": block_device_mapping_v2})
        if metadata:
            attrs.update({"metadata": metadata})
        if networks:
            attrs.update({"networks": networks})
        if personality:
            attrs.update({"personality": personality})
        if admin_pass:
            attrs.update({"admin_pass": admin_pass})
        return self._create(_server.Server, **attrs)

    def delete_server(self, server, ignore_missing=False, force=False):
        """Delete a server

        :param server: The value can be either the ID of a server or a
                       :class:`~ecl.compute.v2.server.Server` instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the server does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent server
        :param bool force: When set to ``True``, the server deletion will be
                           forced immediatly.
        :returns: ``None``
        """
        if force:
            server = self._get_resource(_server.Server, server)
            server.force_delete(self.session)
        else:
            self._delete(_server.Server, server, ignore_missing=ignore_missing)

    def find_server(self, name_or_id, ignore_missing=False):
        """Find a single server

        :param string name_or_id: The name or ID of a server.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: :class:`~ecl.compute.v2.server.Server` or None
        """
        return self._find(_server.Server, name_or_id,
                          ignore_missing=ignore_missing)

    def get_server(self, server):
        """Get a single server

        :param server: The value can be the ID of a server or a
                       :class:`~ecl.compute.v2.server.Server` instance.

        :returns: :class:`~ecl.compute.v2.server.Server`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_server.Server, server)

    def update_server(self, server, **body):
        """Update a server

        :param server: Either the ID of a server or a
                       :class:`~ecl.compute.v2.server.Server` instance.
        :param string name: Server name
        :param string access_ipv4: IPv4 address
        :param string access_ipv6: IPv6 address

        :returns: :class:`~ecl.compute.v2.server.Server`
        """
        return self._update(_server.Server, server, **body)

    def wait_for_server(self, server, status='ACTIVE', failures=['ERROR'],
                        interval=2, wait=120):
        """Not supported
        """
        return resource2.wait_for_status(self.session, server, status,
                                         failures, interval, wait)

    def create_image_from_server(self, server, name, metadata=None):
        """Create image from a certain server

        :param server: Either the ID of a server or a
                       :class:`~ecl.compute.v2.server.Server` instance.
        :param string name: Image name (1-255 characters).
        :param dict metadata: Image metadata ({"metadata_key": "metadata_value"})
        :returns: ``None``
        """
        virtual_server = self.get_server(server)
        return virtual_server.create_image(self.session, name, metadata)

    def get_server_console(self, server, vnc_type):
        """Get the console link of server

        :param server: Either the ID of a server or a
                       :class:`~ecl.compute.v2.server.Server` instance.
        :param vnc_type: should be one of these: novnc, rdp-html5, spice-html5, serial
        :return: console link and type info
        :rtype: :class:`~dict {"url": "", "type": ""}`
        """
        virtual_server = self.get_server(server)
        return virtual_server.get_console(self.session, vnc_type)

    def start_server(self, server):
        """Start the server

        :param server: Either the ID of a server or a
                       :class:`~ecl.compute.v2.server.Server` instance.
        :return: <Response 202>
        """
        virtual_server = self.get_server(server)
        return virtual_server.start(self.session)

    def stop_server(self, server):
        """Stop the server

        :param server: Either the ID of a server or a
                       :class:`~ecl.compute.v2.server.Server` instance.
        :return: <Response 202>
        """
        virtual_server = self.get_server(server)
        return virtual_server.stop(self.session)

    def resize_server(self, server, flavor_id):
        """Resize the server to flavor reference

        :param server: Either the ID of a server or a
                       :class:`~ecl.compute.v2.server.Server` instance.
        :param string flavor_id: ID of flavor to resize
        :return: <Response 202>
        """
        virtual_server = self.get_server(server)
        return virtual_server.resize(self.session, flavor_id)

    def get_server_metadata(self, server):
        """Return a dictionary of metadata for a server

        :param server: Either the ID of a server or a
                       :class:`~ecl.compute.v2.server.Server` or
                       :class:`~ecl.compute.v2.server.ServerDetail`
                       instance.

        :returns: A :class:`~ecl.compute.v2.server.Server` with only the
                  server's metadata. All keys and values are Unicode text.
        """
        res = self.get_server(server)
        metadata = res.get_metadata(self.session)
        result = _server.Server.existing(id=res.id, metadata=metadata)
        return result

    def set_server_metadata(self, server, **metadata):
        """Update metadata for a server

        :param server: Either the ID of a server or a
                       :class:`~ecl.compute.v2.server.Server` or
                       :class:`~ecl.compute.v2.server.ServerDetail`
                       instance.
        :param kwargs metadata: Key/value pairs to be updated in the server's
                                metadata. No other metadata is modified
                                by this call. All keys and values are stored
                                as Unicode.
        :returns: A :class:`~ecl.compute.v2.server.Server` with only the
                  server's metadata. All keys and values are Unicode text.
        """
        res = self.get_server(server)
        metadata = res.set_metadata(self.session, **metadata)
        result = _server.Server.existing(id=res.id, metadata=metadata)
        return result

    def delete_server_metadata(self, server, keys):
        """Delete metadata for a server

        Note: This method will do a HTTP DELETE request for every key in keys.

        :param server: Either the ID of a server or a
                       :class:`~ecl.compute.v2.server.Server` or
                       :class:`~ecl.compute.v2.server.ServerDetail`
                       instance.
        :param array keys: The keys to delete
        :returns: ``None``
        """
        res = self.get_server(server)
        return res.delete_metadata(self.session, keys)

    def create_server_interface(self, server, net_id=None, ip_address=None,
                                port_id=None, fixed_ips=None):
        """Create a new server interface from attributes

        :param server: The server can be either the ID of a server or a
                       :class:`~ecl.compute.v2.server.Server` instance
                       that the interface belongs to.
        :param string net_id: ID of network, may need to specify ip address
                        if
        :param string ip_address: ip_address of add interface to the VM instance
        :param string port_id: ID of port of add interface to the VM instance
        :param dict fixed_ips: dict of fixed ips to add to the VM instance
        :returns: :class:`~ecl.compute.v2.server_interface.ServerInterface`
        """
        attrs = {}
        if net_id is not None:
            attrs.update({"net_id": net_id})
        if ip_address is not None:
            attrs.update({
                "fixed_ips": {
                    "ip_address": ip_address
                }
            })
        if port_id is not None:
            attrs.update({"port_id": port_id})
        if fixed_ips is not None:
            attrs.update({"fixed_ips": fixed_ips})
        server_id = resource2.Resource._get_id(server)
        return self._create(_server_interface.ServerInterface,
                            server_id=server_id, **attrs)

    def delete_server_interface(self, server_interface, server=None,
                                ignore_missing=False):
        """Delete a server interface

        :param server_interface:
            The value can be either the ID of a server interface or a
            :class:`~ecl.compute.v2.server_interface.ServerInterface`
            instance.
        :param server: This parameter need to be specified when ServerInterface
                       ID is given as value. It can be either the ID of a
                       server or a :class:`~ecl.compute.v2.server.Server`
                       instance that the interface belongs to.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the server interface does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent server interface.
        :returns: ``None``
        """
        server_id = self._get_uri_attribute(server_interface, server,
                                            "server_id")
        server_interface = resource2.Resource._get_id(server_interface)

        self._delete(_server_interface.ServerInterface,
                     port_id=server_interface,
                     server_id=server_id,
                     ignore_missing=ignore_missing)

    def server_interfaces(self, server):
        """Return a list of server interfaces

        :param server: The server can be either the ID of a server or a
                       :class:`~ecl.compute.v2.server.Server`.
        :returns: A list of
            :class:`~ecl.compute.v2.server_interface.ServerInterface`
        """
        server_id = resource2.Resource._get_id(server)
        return list(self._list(_server_interface.ServerInterface,
                               paginated=False,
                               server_id=server_id))

    def server_actions(self, server):
        """Return a list of server actions

        :param server: The server can be either the ID of a server or a
                       :class:`~ecl.compute.v2.server.Server`.
        :returns: A list of :class:`~ecl.compute.v2.server_action.ServerAction`
        """
        server_id = resource2.Resource._get_id(server)
        return list(self._list(_server_action.ServerAction, paginated=False,
                    instance_uuid=server_id))

    def get_server_action(self, server_action, server=None):
        """Get a single server action

        :param server_action:
            The value can be the request ID of a server action or a
            :class:`~ecl.compute.v2.server_action.ServerAction`
            instance.
        :param server: This parameter need to be specified when ServerAction
                       ID is given as value. It can be either the ID of a
                       server or a :class:`~ecl.compute.v2.server.Server`
                       instance that the action belongs to.
        :returns: Server Action object
            :class:`~ecl.compute.v2.server_action.ServerAction`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        server = self.get_server(server)

        action = resource2.Resource._get_id(server_action)
        return self._get(_server_action.ServerAction, action,
                         instance_uuid=server.id, )

    def server_volumes(self, server):
        """Return a list of server volumes

        :param server: The server can be either the ID of a server or a
                       :class:`~ecl.compute.v2.server.Server`.
        :returns: A list of :class:`~ecl.compute.v2.server_volume.ServerVolume`
        """
        server_id = resource2.Resource._get_id(server)
        return list(self._list(_server_volume.ServerVolume, paginated=False,
                               serverId=server_id))

    def create_server_volume(self, server, volume_id, device=None):
        """Attaches a volume to the specified server

        :param server: The server can be either the ID of a server or a
                       :class:`~ecl.compute.v2.server.Server`
        :param string volume_id: Volume ID to be attached
        :param string device: Device name that satisfies the following
            conditions: ^/dev/x{0,1}[a-z]{0,1}d{0,1})([a-z]+)[0-9]*$
        """
        attrs = {
            "volumeId": volume_id
        }
        if device:
            attrs.update({"device": device})

        server_id = resource2.Resource._get_id(server)
        return self._create(_server_volume.ServerVolume,
                            serverId=server_id, **attrs)

    def delete_server_volume(self, server_volume, server=None,
                                ignore_missing=False):
        """Detach a volume from a server

        :param server_volume:
            The value can be either the ID of a server volume or a
            :class:`~ecl.compute.v2.server_volume.ServerVolume`
            instance.
        :param server: This parameter need to be specified when ServerVolume
                       ID is given as value. It can be either the ID of a
                       server or a :class:`~ecl.compute.v2.server.Server`
                       instance that the volume belongs to.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the server volume does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent server volume.
        :returns: ``None``
        """
        server_id = self._get_uri_attribute(server_volume, server,
                                            "server_id")
        attachment = resource2.Resource._get_id(server_volume)

        self._delete(_server_volume.ServerVolume,
                     id=attachment,
                     serverId=server_id,
                     ignore_missing=ignore_missing)

    def extensions(self):
        """Retrieve a list of extensions

        :returns: A list of :class:`~ecl.compute.v2.extension.Extension`.
        """
        return list(self._list(extension.Extension, paginated=False))

    def find_flavor(self, name_or_id, ignore_missing=False):
        """Find a single flavor

        :param string name_or_id: The name or ID of a flavor.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: :class:`~ecl.compute.v2.flavor.Flavor` or None
        """
        return self._find(_flavor.Flavor, name_or_id,
                          ignore_missing=ignore_missing)

    def get_flavor(self, flavor):
        """Get a single flavor

        :param flavor: The value can be the ID of a flavor or a
                       :class:`~ecl.compute.v2.flavor.Flavor` instance.

        :returns: :class:`~ecl.compute.v2.flavor.Flavor`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_flavor.Flavor, flavor)

    def flavors(self, details=True):
        """Return a list of flavors

        :param bool details: When ``True``, returns
            :class:`~ecl.compute.v2.flavor.FlavorDetail` objects,
            otherwise :class:`~ecl.compute.v2.flavor.Flavor`.
        :returns: A list of :class:`~ecl.compute.v2.flavor.Flavor`
        """
        flv = _flavor.FlavorDetail if details else _flavor.Flavor
        return list(self._list(flv, paginated=True))

    def delete_image(self, image, ignore_missing=False):
        """Delete an image

        :param image: The value can be either the ID of an image or a
                      :class:`~ecl.compute.v2.image.Image` instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the image does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent image.
        :returns: ``None``
        """
        self._delete(_image.Image, image, ignore_missing=ignore_missing)

    def find_image(self, name_or_id, ignore_missing=False):
        """Find a single image

        :param string name_or_id: The name or ID of a image.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: :class:`~ecl.compute.v2.image.Image` or None
        """
        return self._find(image.Image, name_or_id,
                          ignore_missing=ignore_missing)

    def get_image(self, image):
        """Get a single image

        :param image: The value can be the ID of an image or a
                      :class:`~ecl.compute.v2.image.Image` instance.

        :returns: :class:`~ecl.compute.v2.image.Image`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_image.Image, image)

    def images(self, details=True, **query):
        """Return a list of images

        :param bool details: When ``True``, returns
            :class:`~ecl.compute.v2.image.ImageDetail` objects,
            otherwise :class:`~ecl.compute.v2.image.Image`.
        :param kwargs \*\*query: Optional query parameters to be sent to limit
                                 the resources being returned.
        :returns: A list of :class:`~ecl.compute.v2.image.Image`
        """
        img = _image.ImageDetail if details else _image.Image
        return list(self._list(img, paginated=True, **query))

    def _get_base_resource(self, res, base):
        # Metadata calls for Image and Server can work for both those
        # resources but also ImageDetail and ServerDetail. If we get
        # either class, use it, otherwise create an instance of the base.
        if isinstance(res, base):
            return res
        else:
            return base(id=res)

    def get_image_metadata(self, image):
        """Return a dictionary of metadata for an image

        :param image: Either the ID of an image or a
                       :class:`~ecl.compute.v2.image.Image` or
                       :class:`~ecl.compute.v2.image.ImageDetail`
                       instance.

        :returns: A :class:`~ecl.compute.v2.image.Image` with only the
                  image's metadata. All keys and values are Unicode text.
        :rtype: :class:`~ecl.compute.v2.image.Image`
        """
        res = self._get_base_resource(image, _image.Image)
        metadata = res.get_metadata(self.session)
        result = _image.Image.existing(id=res.id, metadata=metadata)
        return result

    def set_image_metadata(self, image, **metadata):
        """Update metadata for an image

        :param image: Either the ID of an image or a
                       :class:`~ecl.compute.v2.image.Image` or
                       :class:`~ecl.compute.v2.image.ImageDetail`
                       instance.
        :param kwargs metadata: Key/value pairs to be updated in the image's
                                metadata. No other metadata is modified
                                by this call. All keys and values are stored
                                as Unicode.
        :returns: A :class:`~ecl.compute.v2.image.Image` with only the
                  image's metadata. All keys and values are Unicode text.
        """
        res = self._get_base_resource(image, _image.Image)
        metadata = res.set_metadata(self.session, **metadata)
        result = _image.Image.existing(id=res.id, metadata=metadata)
        return result

    def delete_image_metadata(self, image, keys):
        """Delete metadata for an image

        :param image: Either the ID of an image or a
                       :class:`~ecl.compute.v2.image.Image` or
                       :class:`~ecl.compute.v2.image.ImageDetail`
                       instance.
        :param array keys: The keys to delete.
        :rtype: ``None``
        """
        res = self._get_base_resource(image, _image.Image)
        return res.delete_metadata(self.session, keys)

    def create_keypair(self, name=None, public_key=None):
        """Create a new keypair from attributes

        :param string name: The name to associate with the keypair.
        :param string public_key: The public ssh key to import.
            If not provided, a key is generated.
        :returns: :class:`~ecl.compute.v2.keypair.Keypair`
        """
        body = {}
        if name is not None:
            body.update({"name": name})
        if public_key is not None:
            body.update({"public_key": public_key})
        return self._create(_keypair.Keypair, **body)

    def delete_keypair(self, keypair, ignore_missing=False):
        """Delete a keypair

        :param keypair: The value can be either the ID of a keypair or a
                        :class:`~ecl.compute.v2.keypair.Keypair`
                        instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the keypair does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent keypair.
        :returns: ``None``
        """
        self._delete(_keypair.Keypair, keypair, ignore_missing=ignore_missing)

    def get_keypair(self, keypair):
        """Get a single keypair

        :param keypair: The value can be the ID of a keypair or a
                        :class:`~ecl.compute.v2.keypair.Keypair`
                        instance.

        :returns: :class:`~ecl.compute.v2.keypair.Keypair`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_keypair.Keypair, keypair)

    def find_keypair(self, name_or_id, ignore_missing=False):
        """Find a single keypair

        :param string name_or_id: The name or ID of a keypair.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: :class:`~ecl.compute.v2.keypair.Keypair` or None
        """
        return self._find(_keypair.Keypair, name_or_id,
                          ignore_missing=ignore_missing)

    def keypairs(self):
        """Return a list of keypairs

        :returns: A list of keypair objects
        :rtype: :class:`~ecl.compute.v2.keypair.Keypair`
        """
        return list(self._list(_keypair.Keypair, paginated=False))

    def get_limits(self):
        """Retrieve limits that are applied to the project's account

        :returns: A Limits object, including both
                  :class:`~ecl.compute.v2.limits.AbsoluteLimits` and
                  :class:`~ecl.compute.v2.limits.RateLimits`
        :rtype: :class:`~ecl.compute.v2.limits.Limits`
        """
        return self._get(limits.Limits)

    def availability_zones(self, details=False):
        """Return a list of availability zones

        :param bool details: Return extra details about the availability
            zones. This defaults to `False` as it generally
            requires extra permission.
        :returns: A list of :class:`~ecl.compute.v2.availability_zone.AvailabilityZone`
        """
        if details:
            az = _availability_zone.AvailabilityZoneDetail
        else:
            az = _availability_zone.AvailabilityZone
        return list(self._list(az, paginated=False))

    def find_availability_zone(self, name_or_id, ignore_missing=False):
        """Find a single availability_zone

        :param string name_or_id: The name or ID of a availability_zone.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: :class:`~ecl.compute.v2.availability_zone.AvailabilityZone` or None
        """
        return self._find(_availability_zone.AvailabilityZone, name_or_id,
                          ignore_missing=ignore_missing)

    def get_quota(self, tenant_id):
        """Get quota info of a tenant

        :param tenant_id: The ID for the tenant for which you want to show
                        quotas. This ID is different from the tenant ID of authentication.
                        That ID is for the admin tenant.

        :returns: :class:`~ecl.compute.v2.quota.Quota`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_quota.Quota, tenant_id)

    def get_default_quota(self, tenant_id):
        """Get default quota info of a tenant

        :param string tenant_id: The ID for the tenant for which you want to show
                        quotas. This ID is different from the tenant ID of authentication.
                        That ID is for the admin tenant.
        :returns: :class:`~ecl.compute.v2.quota.DefaultQuota`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_quota.DefaultQuota, tenant_id=tenant_id)

    def get_tenant_usage(self, tenant_id):
        """Get tenant usage information of a tenant

        :param string tenant_id: The ID for the tenant for which you want to show
                        usage information. This ID is different from the tenant ID of authentication.
                        That ID is for the admin tenant.

        :returns: :class:`~ecl.compute.v2.quota.TenantUsage`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_quota.TenantUsage, tenant_id)

    def volumes(self, details=True):
        """Return a list of volumes

        :param bool details: Return extra details about the volumes
                             This defaults to `False` as it generally
                             requires extra permission.
        :returns: A list of :class:`~ecl.compute.v2.volume.Volume`
        """
        if details:
            vol = _volume.Volume
        else:
            vol = _volume.VolumeDetail

        return list(self._list(vol, paginated=False))

    def get_volume(self, volume):
        """Get a single volume

        :param volume: The value can be the ID of a volume or a
                       :class:`~ecl.compute.v2.volume.Volume` instance.

        :returns: :class:`~ecl.compute.v2.volume.Volume`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_volume.Volume, volume)

    def create_volume(self, size, name=None, description=None,
                      volume_type=None, metadata=None, availability_zone=None,
                      snapshot_id=None):
        """Create a single volume

        :param size: size of volume to create.
        :param name: display name of volume to create.
        :param description: display description of volume to create.
        :param volume_type: volume type of volume to create.
        :param metadata: size of metadata to create.
        :param availability_zone: availability zone of volume to create.
        :param snapshot_id: ID of snapshot to create from.
        :returns: :class:`~ecl.compute.v2.volume.Volume`
        """
        body = {"size": size}
        if name:
            body.update({"name": name})
        if description:
            body.update({"description": description})
        if volume_type:
            body.update({"volume_type": volume_type})
        if metadata:
            body.update({"metadata": metadata})
        if availability_zone:
            body.update({"availability_zone": availability_zone})
        if snapshot_id:
            body.update({"snapshot_id": snapshot_id})
        return self._create(_volume.Volume, **body)

    def delete_volume(self, volume, ignore_missing=False):
        """Delete an volume

        :param volume: The value can be either the ID of an volume or a
                      :class:`~ecl.compute.v2.volume.Volume` instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the volume does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent volume.
        :returns: ``None``
        """
        self._delete(_volume.Volume, volume, ignore_missing=ignore_missing)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six
from ecl.tests.functional import base

class TestVersion(base.BaseFunctionalTest):

    def test_01_list_versions_information(self):
        versions = list(self.conn.rca.list_versions())
        version = versions[0]
        self.assertIsInstance(version.status, six.string_types)
        self.assertIsInstance(version.id, six.string_types)
        self.assertIsInstance(version.links, list)

    def test_02_show_version(self):
        version = self.conn.rca.get_version()
        self.assertIsInstance(version.status, six.string_types)
        self.assertIsInstance(version.id, six.string_types)
        self.assertIsInstance(version.links, list)



# -*- coding: utf-8 -*-

import six

from ecl.tests.functional import base
from ecl.block_store.v2.limit import Limit


class TestLimits(base.BaseFunctionalTest):

    def test_show(self):
        sot = self.conn.block_store.limits()
        self.assertIsInstance(sot, Limit)



# -*- coding: utf-8 -*-

from ecl import service_filter


class VirtualNetworkApplianceService(service_filter.ServiceFilter):
    """The virtual network appliance service."""

    valid_versions = [service_filter.ValidVersion('v1')]

    def __init__(self, version=None):
        """Create a virtual network appliance service."""
        super(VirtualNetworkApplianceService, self).__init__(
            service_type='virtual-network-appliance',
            version=version
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid
import six
from ecl.tests.functional import base


class TestKeypair(base.BaseFunctionalTest):

    @classmethod
    def test_01_import(cls):
        cls.NAME = uuid.uuid4().hex
        keypair = cls.conn.baremetal.create_keypair(name=cls.NAME)
        print keypair
        assert isinstance(keypair.fingerprint, six.string_types)
        assert isinstance(keypair.name, six.string_types)
        assert isinstance(keypair.public_key, six.string_types)
        assert isinstance(keypair.private_key, six.string_types)

    @classmethod
    def test_02_list(cls):
        keypairs = list(cls.conn.baremetal.keypairs())
        for keypair in keypairs:
            assert isinstance(keypair.fingerprint, six.string_types)
            assert isinstance(keypair.name, six.string_types)
            assert isinstance(keypair.public_key, six.string_types)


    def test_show_03(self):
        keypair = self.conn.baremetal.get_keypair(self.NAME)
        self.assertIsInstance(keypair.public_key, six.string_types)
        self.assertIsInstance(keypair.fingerprint, six.string_types)
        self.assertIsInstance(keypair.name, six.string_types)

    def test_03_delete(self):
        self.conn.baremetal.delete_keypair(self.NAME)

    @classmethod
    def test_05_create(cls):
        cls.NAME2 = uuid.uuid4().hex
        keypair = cls.conn.baremetal.create_keypair(
            name=cls.NAME2, public_key="ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAAAQQCtzdf5vKNNSoeMfTUUj65eJLMjXfIbtc2GQn6+EEISHX6vjBzsTMdToQJEhgg+5rYlb5tc2mvPYNbPDIJV8OyV")
        print keypair
        assert isinstance(keypair.fingerprint, six.string_types)
        assert isinstance(keypair.name, six.string_types)
        assert isinstance(keypair.public_key, six.string_types)

    def test_06_delete(self):
        self.conn.baremetal.delete_keypair(self.NAME2)




# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.block_store import block_store_service
from ecl import resource


class Type(resource.Resource):
    resource_key = "volume_type"
    resources_key = "volume_types"
    base_path = "/types"
    service = block_store_service.BlockStoreService()

    # capabilities
    allow_list = True

    # Properties
    #: A ID representing this type.
    id = resource.prop("id")
    #: Name of the type.
    name = resource.prop("name")
    #: A dict of extra specifications. "capabilities" is a usual key.
    extra_specs = resource.prop("extra_specs", type=dict)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.network import network_service
from ecl import resource2


class FirewallAction(resource2.Resource):
    service = network_service.NetworkService("v2.0")
    base_path = ('/' + service.version +
                 "/firewalls/%(firewall_id)s/%(action)s")

    # Capabilities
    allow_get = True
    allow_create = True

    # Properties
    #: New password reset
    new_password = resource2.Body("new_password")
    #: Username of the firewall
    username = resource2.Body("username")

    def reboot(self, session, firewall_id, type):
        """Reboot firewall."""

        uri = self.base_path % {"firewall_id": firewall_id,
                                "action": "reboot"}
        resp = session.post(
            uri, endpoint_filter=self.service,
            json={"type": type}
        )
        self._translate_response(resp, has_body=False)
        return self

    def reset_password(self, session, firewall_id, username):
        """Reset password of firewall instance."""

        uri = self.base_path % {"firewall_id": firewall_id,
                                "action": "reset_password"}
        resp = session.post(
            uri, endpoint_filter=self.service,
            json={"username": username}
        )
        self._translate_response(resp, has_body=True)
        return self


# -*- coding: utf-8 -*-

import base
from ecl import utils
from ecl import resource2
from ecl.network import network_service


class LoadBalancerInterface(base.NetworkBaseResource):

    resources_key = "load_balancer_interfaces"
    resource_key = "load_balancer_interface"
    service = network_service.NetworkService("v2.0")
    base_path = '/' + service.version + "/load_balancer_interfaces"

    _query_mapping = resource2.QueryParameters(
        "description", "id", "ip_address", "load_balancer_id",
        "name", "network_id", "slot_number", "status", "tenant_id",
        "virtual_ip_address", "sort_key", "sort_dir",
    )

    # Capabilities
    allow_list = True
    allow_get = True
    allow_update = True

    # Properties
    #: Description of the Load Balancer Interface.
    description = resource2.Body("description")
    #: Unique ID of the Load Balancer Interface.
    id = resource2.Body("id")
    #: IP Address
    ip_address = resource2.Body("ip_address")
    #: The ID of load_balancer this load_balancer_interface belongs to.
    load_balancer_id = resource2.Body("load_balancer_id")
    #: Name of the Load Balancer
    name = resource2.Body("name")
    #: Network
    network_id = resource2.Body("network_id")
    #: Slot Number
    slot_number = resource2.Body("slot_number")
    #: The load_balancer status.
    status = resource2.Body("status")
    #: Tenant ID of the owner (UUID)
    tenant_id = resource2.Body("tenant_id")
    #: Virtual IP Address
    virtual_ip_address = resource2.Body("virtual_ip_address")
    #: Properties used for virtual IP address.
    #: virtual_ip_properties.protocol: Redundancy Protocol.
    #: virtual_ip_properties.vrid: VRRP group identifier.
    virtual_ip_properties = resource2.Body("virtual_ip_properties")

    def update(self, session, load_balancer_id,
               load_balancer_interface_id, **attrs):
        """Update a load balancer interface."""
        update_path = '/' + self.service.version
        uri = utils.urljoin(update_path, 'load_balancers',
                            load_balancer_id,
                            'load_balancer_interfaces',
                            load_balancer_interface_id)
        resp = session.put(
            uri, endpoint_filter=self.service,
            json=attrs
        )
        self._translate_response(resp, has_body=True)
        return self


# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from ecl.network.v2 import subnet

IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'allocation_pools': '1',
    'cidr': '2',
    'dns_nameservers': '3',
    'enable_dhcp': True,
    'gateway_ip': '5',
    'host_routes': '6',
    'id': IDENTIFIER,
    'ip_version': '8',
    'ipv6_address_mode': 'dhcpv6-stateless',
    'ipv6_ra_mode': 'radvd',
    'name': '9',
    'network_id': '10',
    'tenant_id': '11',
    'subnetpool_id': '12',
    'description': '13',
    'created_at': '2016-03-09T12:14:57.233772',
    'updated_at': '2016-07-09T12:14:57.233772',
    'segment_id': '16',
}


class TestSubnet(testtools.TestCase):

    def test_basic(self):
        sot = subnet.Subnet()
        self.assertEqual('subnet', sot.resource_key)
        self.assertEqual('subnets', sot.resources_key)
        self.assertEqual('/subnets', sot.base_path)
        self.assertEqual('network', sot.service.service_type)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_get)
        self.assertTrue(sot.allow_update)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = subnet.Subnet(**EXAMPLE)
        self.assertEqual(EXAMPLE['allocation_pools'], sot.allocation_pools)
        self.assertEqual(EXAMPLE['cidr'], sot.cidr)
        self.assertEqual(EXAMPLE['dns_nameservers'], sot.dns_nameservers)
        self.assertTrue(sot.enable_dhcp)
        self.assertEqual(EXAMPLE['gateway_ip'], sot.gateway_ip)
        self.assertEqual(EXAMPLE['host_routes'], sot.host_routes)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['ip_version'], sot.ip_version)
        self.assertEqual(EXAMPLE['ipv6_address_mode'], sot.ipv6_address_mode)
        self.assertEqual(EXAMPLE['ipv6_ra_mode'], sot.ipv6_ra_mode)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['network_id'], sot.network_id)
        self.assertEqual(EXAMPLE['tenant_id'], sot.project_id)
        self.assertEqual(EXAMPLE['description'], sot.description)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.sss import sss_service
from ecl import resource2

class Api_key(resource2.Resource):
    resource_key = None
    resources_key = None
    base_path = '/keys'
    service = sss_service.SssService()

    # Capabilities
    allow_update = True

    # Properties
    #: User's id whose API key updated.
    user_id = resource2.Body('user_id', alternate_id=True)
    #: New API key(keystone username)
    keystone_name = resource2.Body('keystone_name')
    #: New API secret(keystone password)
    keystone_password = resource2.Body('keystone_password')
    #: Update status. 'Success' for successful update
    status = resource2.Body('status')

    def update(self, session, user_id):
        url = '/keys/%s' % user_id
        resp = session.put(url, endpoint_filter=self.service)
        self._translate_response(resp, has_body=True)
        return self



# -*- coding: utf-8 -*-


from ecl.connectivity import connectivity_service
from ecl import resource


class Version(resource.Resource):
    resource_key = 'version'
    resources_key = 'versions'
    base_path = '/'
    service = connectivity_service.ConnectivityService(
        version=connectivity_service.ConnectivityService.UNVERSIONED
    )

    # capabilities
    allow_list = True

    # Properties
    links = resource.prop('links')
    status = resource.prop('status')
    updated = resource.prop('updated')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from ecl.baremetal.v2 import flavor

IDENTIFIER = 'IDENTIFIER'
BASIC_EXAMPLE = {
    'id': IDENTIFIER,
    'links': '2',
    'name': '3',
    'disk': 4,
    'ram': 6,
    'vcpus': 7,
}


class TestFlavor(testtools.TestCase):

    def test_basic(self):
        sot = flavor.Flavor()
        self.assertEqual('flavor', sot.resource_key)
        self.assertEqual('flavors', sot.resources_key)
        self.assertEqual('/flavors', sot.base_path)
        self.assertEqual('baremetal-server', sot.service.service_type)
        self.assertFalse(sot.allow_create)
        self.assertTrue(sot.allow_get)
        self.assertFalse(sot.allow_delete)
        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_update)

    def test_make_basic(self):
        sot = flavor.Flavor(**BASIC_EXAMPLE)
        self.assertEqual(BASIC_EXAMPLE['id'], sot.id)
        self.assertEqual(BASIC_EXAMPLE['links'], sot.links)
        self.assertEqual(BASIC_EXAMPLE['name'], sot.name)
        self.assertEqual(BASIC_EXAMPLE['disk'], sot.disk)
        self.assertEqual(BASIC_EXAMPLE['ram'], sot.ram)
        self.assertEqual(BASIC_EXAMPLE['vcpus'], sot.vcpus)

    def test_detail(self):
        sot = flavor.FlavorDetail()
        self.assertEqual('flavor', sot.resource_key)
        self.assertEqual('flavors', sot.resources_key)
        self.assertEqual('/flavors/detail', sot.base_path)
        self.assertEqual('baremetal-server', sot.service.service_type)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_get)
        self.assertFalse(sot.allow_update)
        self.assertFalse(sot.allow_delete)
        self.assertTrue(sot.allow_list)



# -*- coding: utf-8 -*-

import base
from ecl.virtual_network_appliance import virtual_network_appliance_service
from ecl import resource2


class VirtualNetworkAppliancePlan(base.VirtualNetworkApplianceBaseResource):
    resources_key = "virtual_network_appliance_plans"
    resource_key = "virtual_network_appliance_plan"
    service = virtual_network_appliance_service.\
        VirtualNetworkApplianceService("v1.0")
    base_path = '/' + service.version + '/virtual_network_appliance_plans'

    # Capabilities
    allow_list = True
    allow_get = True

    _query_mapping = resource2.QueryParameters("details")

    # Properties
    #: It identifies connection resource uniquely.
    id = resource2.Body('id')
    #: Name of virtual network appliance plan.
    name = resource2.Body('name')
    #: Description of virtual network appliance plan.
    description = resource2.Body('description')
    #: Appliance type of virtual network appliance plan.
    appliance_type = resource2.Body('appliance_type')
    #: Version of virtual network appliance plan.
    version = resource2.Body('version')
    #: Flavor of virtual network appliance plan.
    flavor = resource2.Body('flavor')
    #: Number of interfaces of virtual network appliance plan.
    number_of_interfaces = resource2.Body('number_of_interfaces')
    #: Enable flag of virtual network appliance plan.
    enabled = resource2.Body('enabled')
    #: Max number of allowed address pairs of virtual network appliance plan.
    max_number_of_aap = resource2.Body('max_number_of_aap')
    #: License information of virtual network appliance plan.
    licenses = resource2.Body('licenses')
    #: Available zone/group information of virtual network appliance plan.
    availability_zones = resource2.Body('availability_zones')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock
import testtools

from ecl.telemetry.v2 import alarm_change

IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'alarm_id': 0,
    'detail': '1',
    'event_id': IDENTIFIER,
    'on_behalf_of': '3',
    'project_id': '4',
    'timestamp': '2015-03-09T12:15:57.233772',
    'type': '6',
    'user_id': '7',
}


class TestAlarmChange(testtools.TestCase):

    def test_basic(self):
        sot = alarm_change.AlarmChange()
        self.assertEqual('alarm_change', sot.resource_key)
        self.assertIsNone(sot.resources_key)
        self.assertEqual('/alarms/%(alarm_id)s/history', sot.base_path)
        self.assertEqual('metering', sot.service.service_type)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_retrieve)
        self.assertFalse(sot.allow_update)
        self.assertFalse(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = alarm_change.AlarmChange(EXAMPLE)
        self.assertEqual(IDENTIFIER, sot.id)
        self.assertEqual(EXAMPLE['alarm_id'], sot.alarm_id)
        self.assertEqual(EXAMPLE['detail'], sot.detail)
        self.assertEqual(IDENTIFIER, sot.event_id)
        self.assertEqual(EXAMPLE['on_behalf_of'], sot.on_behalf_of_id)
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)
        self.assertEqual(EXAMPLE['timestamp'], sot.triggered_at)
        self.assertEqual(EXAMPLE['type'], sot.type)
        self.assertEqual(EXAMPLE['user_id'], sot.user_id)

    def test_list(self):
        sess = mock.Mock()
        resp = mock.Mock()
        resp.json = mock.Mock(return_value=[EXAMPLE, EXAMPLE])
        sess.get = mock.Mock(return_value=resp)
        path_args = {'alarm_id': IDENTIFIER}

        found = alarm_change.AlarmChange.list(sess, path_args=path_args)
        first = next(found)
        self.assertEqual(IDENTIFIER, first.id)
        self.assertEqual(EXAMPLE['alarm_id'], first.alarm_id)
        self.assertEqual(EXAMPLE['detail'], first.detail)
        self.assertEqual(IDENTIFIER, first.event_id)
        self.assertEqual(EXAMPLE['on_behalf_of'], first.on_behalf_of_id)
        self.assertEqual(EXAMPLE['project_id'], first.project_id)
        self.assertEqual(EXAMPLE['timestamp'], first.triggered_at)
        self.assertEqual(EXAMPLE['type'], first.type)
        self.assertEqual(EXAMPLE['user_id'], first.user_id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl import exceptions
from ecl import resource2


# The _check_resource decorator is used on BaseProxy methods to ensure that
# the `actual` argument is in fact the type of the `expected` argument.
# It does so under two cases:
# 1. When strict=False, if and only if `actual` is a Resource instance,
#    it is checked to see that it's an instance of the `expected` class.
#    This allows `actual` to be other types, such as strings, when it makes
#    sense to accept a raw id value.
# 2. When strict=True, `actual` must be an instance of the `expected` class.
def _check_resource(strict=False):
    def wrap(method):
        def check(self, expected, actual=None, *args, **kwargs):
            if (strict and actual is not None and not
               isinstance(actual, resource2.Resource)):
                raise ValueError("A %s must be passed" % expected.__name__)
            elif (isinstance(actual, resource2.Resource) and not
                  isinstance(actual, expected)):
                raise ValueError("Expected %s but received %s" % (
                                 expected.__name__, actual.__class__.__name__))

            return method(self, expected, actual, *args, **kwargs)
        return check
    return wrap


class BaseProxy(object):

    def __init__(self, session):
        self.session = session

    def _get_resource(self, resource_type, value, **attrs):
        """Get a resource object to work on

        :param resource_type: The type of resource to operate on. This should
                              be a subclass of
                              :class:`~ecl.resource2.Resource` with a
                              ``from_id`` method.
        :param value: The ID of a resource or an object of ``resource_type``
                      class if using an existing instance, or None to create a
                      new instance.
        :param path_args: A dict containing arguments for forming the request
                          URL, if needed.
        """
        if value is None:
            # Create a bare resource
            res = resource_type.new(**attrs)
        elif not isinstance(value, resource_type):
            # Create from an ID
            res = resource_type.new(id=value, **attrs)
        else:
            # An existing resource instance
            res = value
            res._update(**attrs)

        return res

    def _get_uri_attribute(self, child, parent, name):
        """Get a value to be associated with a URI attribute

        `child` will not be None here as it's a required argument
        on the proxy method. `parent` is allowed to be None if `child`
        is an actual resource, but when an ID is given for the child
        one must also be provided for the parent. An example of this
        is that a parent is a Server and a child is a ServerInterface.
        """
        if parent is None:
            value = getattr(child, name)
        else:
            value = resource2.Resource._get_id(parent)
        return value

    def _find(self, resource_type, name_or_id, ignore_missing=False,
              **attrs):
        """Find a resource

        :param name_or_id: The name or ID of a resource to find.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource2.
        :param dict attrs: Attributes to be passed onto the
                           :meth:`~ecl.resource2.Resource.find`
                           method, such as query parameters.

        :returns: An instance of ``resource_type`` or None
        """
        return resource_type.find(self.session, name_or_id,
                                  ignore_missing=ignore_missing,
                                  **attrs)

    @_check_resource(strict=False)
    def _delete(self, resource_type, value, ignore_missing=False, **attrs):
        """Delete a resource

        :param resource_type: The type of resource to delete. This should
                              be a :class:`~ecl.resource2.Resource`
                              subclass with a ``from_id`` method.
        :param value: The value to delete. Can be either the ID of a
                      resource or a :class:`~ecl.resource2.Resource`
                      subclass.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent resource2.
        :param dict attrs: Attributes to be passed onto the
                           :meth:`~ecl.resource2.Resource.delete`
                           method, such as the ID of a parent resource.

        :returns: The result of the ``delete``
        :raises: ``ValueError`` if ``value`` is a
                 :class:`~ecl.resource2.Resource` that doesn't match
                 the ``resource_type``.
                 :class:`~ecl.exceptions.ResourceNotFound` when
                 ignore_missing if ``False`` and a nonexistent resource
                 is attempted to be deleted.

        """
        res = self._get_resource(resource_type, value, **attrs)

        try:
            rv = res.delete(self.session)
        except exceptions.NotFoundException as e:
            if ignore_missing:
                return None
            else:
                # Reraise with a more specific type and message
                raise exceptions.ResourceNotFound(
                    message="No %s found for %s" %
                            (resource_type.__name__, value),
                    details=e.details, response=e.response,
                    request_id=e.request_id, url=e.url, method=e.method,
                    http_status=e.http_status, cause=e.cause)

        return rv

    @_check_resource(strict=False)
    def _update(self, resource_type, value, **attrs):
        """Update a resource

        :param resource_type: The type of resource to update.
        :type resource_type: :class:`~ecl.resource2.Resource`
        :param value: The resource to update. This must either be a
                      :class:`~ecl.resource2.Resource` or an id
                      that corresponds to a resource2.
        :param dict attrs: Attributes to be passed onto the
                           :meth:`~ecl.resource2.Resource.update`
                           method to be updated. These should correspond
                           to either :class:`~ecl.resource2.Body`
                           or :class:`~ecl.resource2.Header`
                           values on this resource.

        :returns: The result of the ``update``
        :rtype: :class:`~ecl.resource2.Resource`
        """
        res = self._get_resource(resource_type, value, **attrs)
        return res.update(self.session)

    def _create(self, resource_type, **attrs):
        """Create a resource from attributes

        :param resource_type: The type of resource to create.
        :type resource_type: :class:`~ecl.resource2.Resource`
        :param path_args: A dict containing arguments for forming the request
                          URL, if needed.
        :param dict attrs: Attributes to be passed onto the
                           :meth:`~ecl.resource2.Resource.create`
                           method to be created. These should correspond
                           to either :class:`~ecl.resource2.Body`
                           or :class:`~ecl.resource2.Header`
                           values on this resource.

        :returns: The result of the ``create``
        :rtype: :class:`~ecl.resource2.Resource`
        """
        res = resource_type.new(**attrs)
        return res.create(self.session)

    @_check_resource(strict=False)
    def _get(self, resource_type, value=None, requires_id=True, **attrs):
        """Get a resource

        :param resource_type: The type of resource to get.
        :type resource_type: :class:`~ecl.resource2.Resource`
        :param value: The value to get. Can be either the ID of a
                      resource or a :class:`~ecl.resource2.Resource`
                      subclass.
        :param dict attrs: Attributes to be passed onto the
                           :meth:`~ecl.resource2.Resource.get`
                           method. These should correspond
                           to either :class:`~ecl.resource2.Body`
                           or :class:`~ecl.resource2.Header`
                           values on this resource.

        :returns: The result of the ``get``
        :rtype: :class:`~ecl.resource2.Resource`
        """
        res = self._get_resource(resource_type, value, **attrs)

        try:
            return res.get(self.session, requires_id=requires_id)
        except exceptions.NotFoundException as e:
            raise exceptions.ResourceNotFound(
                message="No %s found for %s" %
                        (resource_type.__name__, value),
                details=e.details, response=e.response,
                request_id=e.request_id, url=e.url, method=e.method,
                http_status=e.http_status, cause=e.cause)

    def _list(self, resource_type, value=None, paginated=False, **attrs):
        """List a resource

        :param resource_type: The type of resource to delete. This should
                              be a :class:`~ecl.resource2.Resource`
                              subclass with a ``from_id`` method.
        :param value: The resource to list. It can be the ID of a resource, or
                      a :class:`~ecl.resource2.Resource` object. When set
                      to None, a new bare resource is created.
        :param bool paginated: When set to ``False``, expect all of the data
                               to be returned in one response. When set to
                               ``True``, the resource supports data being
                               returned across multiple pages.
        :param dict attrs: Attributes to be passed onto the
            :meth:`~ecl.resource2.Resource.list` method. These should
            correspond to either :class:`~ecl.resource2.URI` values
            or appear in :data:`~ecl.resource2.Resource._query_mapping`.

        :returns: A generator of Resource objects.
        :raises: ``ValueError`` if ``value`` is a
                 :class:`~ecl.resource2.Resource` that doesn't match
                 the ``resource_type``.
        """
        res = self._get_resource(resource_type, value, **attrs)
        return res.list(self.session, paginated=paginated, **attrs)

    def _head(self, resource_type, value=None, **attrs):
        """Retrieve a resource's header

        :param resource_type: The type of resource to retrieve.
        :type resource_type: :class:`~ecl.resource2.Resource`
        :param value: The value of a specific resource to retreive headers
                      for. Can be either the ID of a resource,
                      a :class:`~ecl.resource2.Resource` subclass,
                      or ``None``.
        :param dict attrs: Attributes to be passed onto the
                           :meth:`~ecl.resource2.Resource.head` method.
                           These should correspond to
                           :class:`~ecl.resource2.URI` values.

        :returns: The result of the ``head`` call
        :rtype: :class:`~ecl.resource2.Resource`
        """
        res = self._get_resource(resource_type, value, **attrs)
        return res.head(self.session)

    def wait_for_status(self, value, status, failures=[], interval=2,
                        wait=120):
        """Wait for a resource to be in a particular status.

        :param value: The resource to wait on to reach the status. The
                      resource must have a status attribute.
        :type value: :class:`~ecl.resource2.Resource`
        :param status: Desired status of the resource2.
        :param list failures: Statuses that would indicate the transition
                              failed such as 'ERROR'.
        :param interval: Number of seconds to wait between checks.
        :param wait: Maximum number of seconds to wait for the change.

        :return: Method returns resource on success.
        :raises: :class:`~ecl.exceptions.ResourceTimeout` transition
                 to status failed to occur in wait seconds.
        :raises: :class:`~ecl.exceptions.ResourceFailure` resource
                 transitioned to one of the failure states.
        :raises: :class:`~AttributeError` if the resource does not have a
                 status attribute
        """
        return resource2.wait_for_status(self.session, value, status,
                                         failures, interval, wait)

    def wait_for_delete(self, value, interval=2, wait=120):
        """Wait for the resource to be deleted.

        :param value: The resource to wait on to be deleted.
        :type value: :class:`~ecl.resource2.Resource`
        :param interval: Number of seconds to wait between checks.
        :param wait: Maximum number of seconds to wait for the delete.

        :return: Method returns resource on success.
        :raises: :class:`~ecl.exceptions.ResourceTimeout` transition
                 to delete failed to occur in wait seconds.
        """
        return resource2.wait_for_delete(self.session, value, interval, wait)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl import utils
from ecl.storage import storage_service

from ecl import resource2
from ecl.storage import storage_service


class Storage(resource2.Resource):
    resource_key = 'virtual_storage'
    resources_key = 'virtual_storages'
    base_path = '/virtual_storages'

    service = storage_service.StorageService()

    # capabilities
    allow_list = True
    allow_get = True
    allow_create = True
    allow_delete = True
    allow_update = True

    # Properties
    #: id of storage(UUID)
    id = resource2.Body('id')
    #: Network id(UUID) of storage
    network_id = resource2.Body('network_id')
    #: Subnet id(UUID) of storage
    subnet_id = resource2.Body('subnet_id')
    #: The pool of IP addresses for storage
    ip_addr_pool = resource2.Body('ip_addr_pool', type=dict)
    #: static routes settings of storage
    host_routes = resource2.Body('host_routes', type=list)
    #: volume type id(UUID) of storage
    volume_type_id = resource2.Body('volume_type_id')
    #: name of storage
    name = resource2.Body('name')
    #: description of storage
    description = resource2.Body('description')
    #: status of storage
    status = resource2.Body('status')
    #: creation timestamp of storage
    created_at = resource2.Body('created_at')
    #: update timestamp of storage
    updated_at = resource2.Body('updated_at')
    #: error description of storage
    error_message = resource2.Body('error_message')

    def create(self, session, **attrs):
        body = {"virtual_storage": attrs}
        resp = session.post(
            self.base_path, endpoint_filter=self.service,
            json=body,
            headers={"Accept": "application/json"}
        )
        self._translate_response(resp, has_body=True)
        return self

    def update(self, session, storage_id, has_body=True, **attrs):
        uri = utils.urljoin(self.base_path, storage_id)
        body = {"virtual_storage": attrs}
        args = {'json': body}
        resp = session.put(uri, endpoint_filter=self.service, **args)
        self._translate_response(resp, has_body)
        return self


class StorageDetail(Storage):
    base_path = '/virtual_storages/detail'
    allow_get = False
    allow_create = False
    allow_delete = False
    allow_update = False



# -*- coding: utf-8 -*-

import base
from ecl.network import network_service
from ecl import resource2


class ColocationLogicalLink(base.NetworkBaseResource):
    resource_key = 'colocation_logical_link'
    resources_key = 'colocation_logical_links'
    service = network_service.NetworkService("v2.0")
    base_path = '/' + service.version + '/colocation_logical_links'


    # capabilities
    allow_create = True
    allow_get = True
    allow_update = True
    allow_delete = True
    allow_list = True

    # query mappings
    _query_mapping = resource2.QueryParameters('bandwidth',
                                               'colocation_physical_link_id',
                                               'description',
                                               'id',
                                               'name',
                                               'network_id',
                                               'status',
                                               'tenant_id',
                                               'type_a_port_id',
                                               'type_b_port_id',
                                               'vlan_id',
                                               "sort_key", "sort_dir",)

    # properties
    #: Colocation Physical Link ID.
    colocation_physical_link_id = resource2.Body('colocation_physical_link_id')
    #: Colocation Logical Link description.
    description = resource2.Body('description')
    #: Colocation Logical Link unique id.
    id = resource2.Body('id')
    #: Colocation Logical Link name.
    name = resource2.Body('name')
    #: Network connected to this link.
    network_id = resource2.Body('network_id')
    #: The Colocation Logical Link status.
    status = resource2.Body('status')
    #: Colocation Logical Link tags.
    tags = resource2.Body('tags')
    #: The owner name of Colocation Logical Link.
    tenant_id = resource2.Body('tenant_id')
    #: Type A Port ID.
    type_a_port_id = resource2.Body('type_a_port_id')
    #: Type B Port ID.
    type_b_port_id = resource2.Body('type_b_port_id')
    #: Logical port vlan id.
    vlan_id = resource2.Body('vlan_id')



# -*- coding: utf-8 -*-

from ecl.provider_connectivity import provider_connectivity_service
from ecl import resource2


class Operation(resource2.Resource):
    resources_key = "operations"
    service = provider_connectivity_service.ProviderConnectivityService("v1.0")
    base_path = '/' + service.version + '/icc/operations'

    # Capabilities
    allow_list = True

    _query_mapping = resource2.QueryParameters("connection_id",)

    # Properties
    #: It identifies connection resource uniquely.
    connection_id = resource2.Body('connection_id', alternate_id=True)
    #: order type.
    order_type = resource2.Body('order_type')
    #: operation phase.
    phase = resource2.Body('phase')
    #: Status of phase.
    operation_status = resource2.Body('operation_status')
    #: reception datetime. YYYY-MM-DD HH:MM:SS format.
    reception_datetime = resource2.Body('reception_datetime')
    #: commit datetime. YYYY-MM-DD HH:MM:SS format.
    commit_datetime = resource2.Body('commit_datetime')
    #: Request body is stored.
    request_body = resource2.Body('request_body')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import unittest

from ecl.tests.functional import base


@unittest.skipUnless(base.service_exists(service_type="metering"),
                     "Metering service does not exist")
class TestStatistics(base.BaseFunctionalTest):

    def test_list(self):
        found_something = False
        for met in self.conn.telemetry.meters():
            try:
                stat = next(self.conn.telemetry.statistics(met))
                self.assertIn('period_end', stat)
                found_something = True
            except Exception:
                pass
        self.assertTrue(found_something)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six

from ecl import utils


class MetadataMixin(object):

    def _metadata(self, method, key=None, clear=False, delete=False,
                  **metadata):
        for k, v in metadata.items():
            if not isinstance(v, six.string_types):
                raise ValueError("The value for %s (%s) must be "
                                 "a text string" % (k, v))

        # If we're in a ServerDetail, we need to pop the "detail" portion
        # of the URL off and then everything else will work the same.
        pos = self.base_path.find("detail")
        if pos != -1:
            base = self.base_path[:pos]
        else:
            base = self.base_path

        if key is not None:
            url = utils.urljoin(base, self.id, "metadata", key)
        else:
            url = utils.urljoin(base, self.id, "metadata")

        kwargs = {"endpoint_filter": self.service}
        if metadata or clear:
            # 'meta' is the key for singular modifications.
            # 'metadata' is the key for mass modifications.
            key = "meta" if key is not None else "metadata"
            kwargs["json"] = {key: metadata}

        headers = {"Accept": ""} if delete else {}

        response = method(url, headers=headers, **kwargs)

        # DELETE doesn't return a JSON body while everything else does.
        return response.json() if not delete else None

    def get_metadata(self, session):
        """Retrieve metadata

        :param session: The session to use for this request.

        :returns: A dictionary of the requested metadata. All keys and values
                  are Unicode text.
        :rtype: dict
        """
        result = self._metadata(session.get)
        return result["metadata"]

    def set_metadata(self, session, **metadata):
        """Update metadata

        This call will replace only the metadata with the same keys
        given here. Metadata with other keys will not be modified.

        :param session: The session to use for this request.
        :param kwargs metadata: key/value metadata pairs to be update on
                                this server instance. All keys and values
                                are stored as Unicode.

        :returns: A dictionary of the metadata after being updated.
                  All keys and values are Unicode text.
        :rtype: dict
        """
        if not metadata:
            return dict()

        result = self._metadata(session.post, **metadata)
        return result["metadata"]

    def delete_metadata(self, session, keys):
        """Delete metadata

        Note: This method will do a HTTP DELETE request for every key in keys.

        :param session: The session to use for this request.
        :param keys: The keys to delete.

        :rtype: ``None``
        """
        for key in keys:
            self._metadata(session.delete, key=key, delete=True)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl import proxy2
from ecl import session
from ecl.rca.v1 import user as _user
from ecl.rca import version as _version

class VersionSession(session.Session):

    def get_endpoint(self, auth=None, interface='public', service_type=None,
                     **kwargs):
        """Override get endpoint to automate endpoint filtering

        This method uses the service catalog to find the root URI of
        each service and then gets all available versions directly
        from the service, not from the service catalog.

        Endpoints are cached per service type and interface combination
        so that they're only requested from the remote service once
        per instance of this class.
        """

        sc_endpoint = super(VersionSession, self).get_endpoint(
            auth, interface, service_type
        )

        if sc_endpoint == None or sc_endpoint == '':
            return sc_endpoint
        segments = sc_endpoint.split('/')
        if len(segments) > 0:
            segments = segments[:-2]
            sc_endpoint = ""
            for segment in segments:
                sc_endpoint += segment + '/'
            sc_endpoint = sc_endpoint[:-1]

        return sc_endpoint


class Proxy(proxy2.BaseProxy):

    def users(self):
        """
        List VPN user. Now, VPN user can create per one tenant, so user
        list hash only one user.

        :return: A list of user objects
        """
        return list(self._list(_user.User))

    def get_user(self, username):
        """
        Show VPN user.

        :param username: Query user name
        :return: One :class:`~ecl.rca.v1.user.User` or
                     :class:`~ecl.exceptions.ResourceNotFound`when no
                     resource can be found.
        """
        return self._get(_user.User, username)

    def create_user(self, password):
        """
        Create VPN user. One VPN user can create per one tenant.
        And username equal to tenant_id.

        :param password: The passowrd of VPN connection.
                Available character is 8-127 character of alphabet[a-zA-Z],
                number[0-9] and Symbols[.-_/*+,!#$%&()~|].
        :return: The results of user creation
        :rtype: :class:`~ecl.rca.v1.user.User`
        """
        return self._create(_user.User, password=password)

    def update_user(self, username, password):
        """
        Update VPN user.

        :param username: The username of VPN user.
        :param password: The passowrd of VPN connection. Available character is
                8-127 character of alphabet[a-zA-Z], number[0-9] and Symbols[
                .-_/*+,!#$%&()~|].
        :return: :class:`~ecl.rca.v1.user.User`
        """
        return self._update(_user.User, name=username, password=password)

    def delete_user(self, username):
        """
        Delete VPN user.

        :param username: The username of VPN user.
        :return: ``None``
        """
        return self._delete(_user.User, username)

    def find_user(self, name_or_id, ignore_missing=False):
        """Find a single VPN user.

        :param name_or_id: The name or ID of a user.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.rca.v1.user.User` or None
        """
        return self._find(_user.User, name_or_id, ignore_missing=ignore_missing)

    def get_version(self):
        """
        Show VPN version.

        :return: :class:`~ecl.rca.version.Version`
        """
        version = _version.Version()
        v_session = VersionSession(
            profile=self.session.profile,
            user_agent=self.session.user_agent,
        )
        for attr, value in self.session.__dict__.iteritems():
            v_session.__setattr__(attr, value)

        return version.get_version(session=v_session)

    def versions(self):
        """
        List VPN versions.

        :return: A generator of version objects
        """
        version = _version.Version()
        v_session = VersionSession(
            profile=self.session.profile,
            user_agent=self.session.user_agent,
        )
        for attr, value in self.session.__dict__.iteritems():
            v_session.__setattr__(attr, value)

        return list(version.list_version(session=v_session))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from ecl.compute.v2 import server
from ecl.tests.functional import base


class TestServer(base.BaseFunctionalTest):
    @classmethod
    def setUpClass(cls):
        super(TestServer, cls).setUpClass()
        instance_name = "test_sdk_server"
        cls.one_server = cls.conn.compute.find_server(instance_name)
        cls.one_serverif = None
        if cls.one_server is None:
            network_name = "test-network"
            cls.network = cls.conn.network.find_network(network_name)
            if cls.network is None:
                cls.conn.network.create_network(
                    name=network_name
                )
            if not cls.network:
                # We can't call TestCase.fail from within the setUpClass
                # classmethod, but we need to raise some exception in order
                # to get this setup to fail and thusly fail the entire class.
                raise Exception("Unable to create network for TestServer")
            subnet_name = "test_subnet"
            cls.subnet = cls.conn.network.find_subnet(subnet_name)
            if cls.subnet is None:
                cls.subnet = cls.conn.network.create_subnet(
                    cls.network.id, "172.16.12.0/24", name=subnet_name
                )
            if not cls.subnet:
                # We can't call TestCase.fail from within the setUpClass
                # classmethod, but we need to raise some exception in order
                # to get this setup to fail and thusly fail the entire class.
                raise Exception("Unable to create subnet for TestServer")

            cls.flavor = "1CPU-4GB"
            cls.image = "ad936ae4-2983-4f23-9187-e47e82cb2725"

            cls.one_server = cls.conn.compute.create_server(
                name=instance_name, flavor_id=cls.flavor, image_id=cls.image,
                networks=[{"uuid": cls.network.id}])
            cls.conn.compute.wait_for_server(cls.one_server)
        assert isinstance(cls.one_server, server.Server)
        cls.assertIs(cls.one_server.name, instance_name)

        ifs = cls.conn.compute.server_interfaces(cls.one_server.id)
        if ifs:
            cls.one_serverif = ifs[0]

    def test_create(self):
        one_server = self.conn.compute.create_server(
            name=self.one_server.name, flavor_id=self.one_server.flavorRef,
            image_id=self.one_server.imageRef,
            networks=[{"uuid": self.network.id}])
        self.conn.compute.wait_for_server(one_server)

    def test_delete(self):
        instance_name = "test_sdk_server"
        one_server = self.conn.compute.find_server(instance_name)
        if one_server is not None:
            self.conn.compute.delete_servers(one_server.id)
            self.conn.compute.wait_for_delete(one_server)

    def test_find(self):
        sot = self.conn.compute.find_server(self.one_server.name)
        self.assertEqual(self.one_server.id, sot.id)

    def test_get(self):
        sot = self.conn.compute.get_server(self.one_server.id)
        self.assertEqual(self.one_server.name, sot.name)
        self.assertEqual(self.one_server.id, sot.id)

    def test_list(self):
        names = [o.name for o in self.conn.compute.servers()]
        self.assertIn(self.one_server.name, names)

    def test_update(self):
        self.conn.compute.update_server(self.one_server.id, name="test_sdk_server")
        self.assertEqual(self.one_server.name, "test_sdk_server")

    def test_get_console(self):
        console = self.conn.compute.get_server_console(self.one_server.id,
                                                       vnc_type="novnc")
        self.assertEqual("novnc", console.get("type"))

    def test_start(self):
        ret = self.conn.compute.start_server(self.one_server.id)
        self.assertIsNotNone(ret)

    def test_stop(self):
        ret = self.conn.compute.stop_server(self.one_server.id)
        self.assertIsNotNone(ret)

    def test_resize(self):
        ret = self.conn.compute.resize_server(self.one_server.id, "2CPU-8GB")
        self.assertIsNotNone(ret)

    def test_server_metadata(self):
        test_server = self.conn.compute.get_server(self.one_server.id)

        # get metadata
        test_server = self.conn.compute.get_server_metadata(test_server)
        self.assertFalse(test_server.metadata)

        # set no metadata
        self.conn.compute.set_server_metadata(test_server)
        test_server = self.conn.compute.get_server_metadata(test_server)
        self.assertFalse(test_server.metadata)

        # set empty metadata
        self.conn.compute.set_server_metadata(test_server, k0='')
        server = self.conn.compute.get_server_metadata(test_server)
        self.assertTrue(server.metadata)

        # set metadata
        self.conn.compute.set_server_metadata(test_server, k1='v1')
        test_server = self.conn.compute.get_server_metadata(test_server)
        self.assertTrue(test_server.metadata)
        self.assertEqual(2, len(test_server.metadata))
        self.assertIn('k0', test_server.metadata)
        self.assertEqual('', test_server.metadata['k0'])
        self.assertIn('k1', test_server.metadata)
        self.assertEqual('v1', test_server.metadata['k1'])

        # set more metadata
        self.conn.compute.set_server_metadata(test_server, k2='v2')
        test_server = self.conn.compute.get_server_metadata(test_server)
        self.assertTrue(test_server.metadata)
        self.assertEqual(3, len(test_server.metadata))
        self.assertIn('k0', test_server.metadata)
        self.assertEqual('', test_server.metadata['k0'])
        self.assertIn('k1', test_server.metadata)
        self.assertEqual('v1', test_server.metadata['k1'])
        self.assertIn('k2', test_server.metadata)
        self.assertEqual('v2', test_server.metadata['k2'])

        # update metadata
        self.conn.compute.set_server_metadata(test_server, k1='v1.1')
        test_server = self.conn.compute.get_server_metadata(test_server)
        self.assertTrue(test_server.metadata)
        self.assertEqual(3, len(test_server.metadata))
        self.assertIn('k0', test_server.metadata)
        self.assertEqual('', test_server.metadata['k0'])
        self.assertIn('k1', test_server.metadata)
        self.assertEqual('v1.1', test_server.metadata['k1'])
        self.assertIn('k2', test_server.metadata)
        self.assertEqual('v2', test_server.metadata['k2'])

        # delete metadata
        self.conn.compute.delete_server_metadata(
            test_server, test_server.metadata.keys())
        test_server = self.conn.compute.get_server_metadata(test_server)
        self.assertFalse(test_server.metadata)

    def test_server_interfaces(self):
        ifs = self.conn.compute.server_interfaces(self.one_server.id)
        self.assertGreaterEqual(len(ifs), 0)

    def test_create_server_interface(self):
        network_list = self.conn.network.networks()
        network_id = None
        if len(network_list) > 0:
            networ_id = network_list[0].id
        server_if = self.conn.compute.create_server_interface(
            self.one_server.id, net_id=network_id)
        self.assertEqual(server_if.server_id, self.one_server.id)
        self.assertEqual(server_if.net_id, network_id)

    def test_delete_server_interface(self):
        ifs = self.conn.compute.server_interfaces(self.one_server.id)
        if_delete = None
        if len(ifs) > 0:
            if_delete = ifs[-1]
        ret = self.conn.compute.delete_server_interface(
            if_delete, self.one_server.id)
        self.assertIsNone(ret)

    def test_server_actions(self):
        actions = self.conn.compute.server_actions(self.one_server.id)
        self.assertGreaterEqual(len(actions), 0)

    def test_get_server_actions(self):
        actions = self.conn.compute.server_actions(self.one_server.id)
        action_id = None
        if len(actions) > 0:
            action_id = actions[0].request_id
        print action_id
        action = self.conn.compute.get_server_action(action_id, self.one_server.id)
        print action
        self.assertEqual(action.request_id, action_id)

    def test_server_volumes(self):
        attachments = self.conn.compute.server_volumes(self.one_server.id)
        self.assertGreaterEqual(len(attachments), 0)

    def test_create_server_volume(self):
        volume_id = "8d015ea7-3c23-4012-8565-236ee67f07f7"
        attachment = self.conn.compute.create_server_volume(
            self.one_server.id, volume_id
        )
        self.assertEqual(attachment.volumeId, volume_id)
        self.assertEqual(attachment.serverId, self.one_server.id)

    def test_delete_server_volume(self):
        attachments = self.conn.compute.server_volumes(self.one_server.id)
        if len(attachments) <= 0:
            return
        attachment = attachments[0]
        ret = self.conn.compute.delete_server_volume(
            attachment.id, self.one_server.id)
        self.assertIsNone(ret)




# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.baremetal import baremetal_service
from ecl import resource2


class NicPhysicalPort(resource2.Resource):
    resource_key = "nic_physical_port"
    resources_key = "nic_physical_ports"
    base_path = '/servers/%(server_id)s/nic_physical_ports'
    service = baremetal_service.BaremetalService()

    # Capabilities
    allow_get = True
    allow_list = True

    # Properties
    #: ID for the specified server.
    server_id = resource2.Body('server_id')
    #: UUID of the NicPhysicalPort.
    id = resource2.Body('id')
    #: MAC address of the NicPhysicalPort.
    mac_addr = resource2.Body('mac_addr')
    #: Network controller port id of physical leaf switch.
    network_physical_port_id = resource2.Body('network_physical_port_id')
    #: Value = 'data'(default) or 'storage'. Assigning preferentially the
    #: order to the NIC physical port. data plane: all devices cloud be
    #: connected. storage plane: this plane type is only used and optimized
    #: for storages between servers.You can assigning each until 2 planes.
    plane = resource2.Body('plane')
    #: Logical attached ports of host.
    attached_ports = resource2.Body('attached_ports')
    #: ID of hardware attaching the NicPhysicalPort.
    hardware_id = resource2.Body('hardware_id')

    def list(self, session, server_id):
        uri = self.base_path % {"server_id" : server_id}
        resp = session.get(uri, endpoint_filter=self.service,
                           headers={"Accept": "application/json"})
        resp = resp.json()
        resp = resp[self.resources_key]

        for data in resp:
            value = self.existing(**data)
            yield value

    def get(self, session, server_id, port_id):
        uri = self.base_path + '/%(nic_physical_port_id)s'
        uri = uri % {"server_id": server_id,
                                "nic_physical_port_id": port_id}
        resp = session.get(uri, endpoint_filter=self.service,
                           headers={"Accept": "application/json"})
        self._translate_response(resp, has_body=True)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging


def enable_logging(debug=False, path=None, stream=None):
    """Enable logging to a file at path and/or a console stream.

    This function is available for debugging purposes. If you wish to
    log this package's message in your application, the standard library
    ``logging`` package will receive these messages in any handlers you
    create.

    :param bool debug: Set this to ``True`` to receive debug messages,
                       which includes HTTP requests and responses,
                       or ``False`` for warning messages.
    :param str path: If a *path* is specified, logging output will
                     written to that file   in addition to sys.stderr.
                     The path is passed to logging.FileHandler,
                     which will append messages the file (and create
                     it if needed).
    :param stream: One of ``None `` or ``sys.stdout`` or ``sys.stderr``.
                   If it is ``None``, nothing is logged to a stream.
                   If it isn't ``None``, console output is logged
                   to this stream.

    :rtype: None
    """
    if path is None and stream is None:
        raise ValueError("path and/or stream must be set")

    logger = logging.getLogger('ecl')
    ksalog = logging.getLogger('keystoneauth')
    formatter = logging.Formatter(
        '%(asctime)s %(levelname)s: %(name)s %(message)s')

    if stream is not None:
        console = logging.StreamHandler(stream)
        console.setFormatter(formatter)
        logger.addHandler(console)
        ksalog.addHandler(console)

    if path is not None:
        file_handler = logging.FileHandler(path)
        file_handler.setFormatter(formatter)
        logger.addHandler(file_handler)
        ksalog.addHandler(file_handler)

    logger.setLevel(logging.DEBUG if debug else logging.WARNING)
    ksalog.setLevel(logging.DEBUG if debug else logging.WARNING)


def urljoin(*args):
    """A custom version of urljoin that simply joins strings into a path.

    The real urljoin takes into account web semantics like when joining a url
    like /path this should be joined to http://host/path as it is an anchored
    link. We generally won't care about that in client.
    """
    return '/'.join(str(a or '').strip('/') for a in args)
